@@ -6,7 +6,8 @@
 ## Copyright (C) 2005 Dimitur Kirov <dkirov@gmail.com>
 ## Copyright (C) 2005 Travis Shirk <travis@pobox.com>
 ## Copyright (C) 2005 Norman Rasmussen <norman@rasmussen.co.za>
-##
+## Copyright (C) 2006 Stefan Bethge <stefan@lanpartei.de>
+## 
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
 ## by the Free Software Foundation; version 2 only.
@@ -18,10 +19,9 @@
 ##
 
 
-import sre
+import re
 import copy
-import i18n
-_ = i18n._
+import defs
 
 
 (
@@ -38,9 +38,12 @@ opt_str = [ 'string', 0 ]
 opt_bool = [ 'boolean', 0 ]
 opt_color = [ 'color', '^(#[0-9a-fA-F]{6})|()$' ]
 opt_one_window_types = ['never', 'always', 'peracct', 'pertype']
+opt_treat_incoming_messages = ['', 'chat', 'normal']
 
 class Config:
 
+	DEFAULT_ICONSET = 'dcraven'
+
 	__options = {
 		# name: [ type, default_value, help_string ]
 		'verbose': [ opt_bool, False, '', True ],
@@ -50,9 +53,10 @@ class Config:
 		'notify_on_signout': [ opt_bool, False ],
 		'notify_on_new_message': [ opt_bool, True ],
 		'autopopupaway': [ opt_bool, False ],
-		'use_notif_daemon': [ opt_bool, True , _('Use DBus and Notification-Daemon to show notifications') ],
+		'use_notif_daemon': [ opt_bool, True , _('Use D-Bus and Notification-Daemon to show notifications') ],
 		'ignore_unknown_contacts': [ opt_bool, False ],
-		'showoffline': [ opt_bool, False, '', True ],
+		'showoffline': [ opt_bool, False ],
+		'show_transports_group': [ opt_bool, True ],
 		'autoaway': [ opt_bool, True ],
 		'autoawaytime': [ opt_int, 5, _('Time in minutes, after which your status changes to away.') ],
 		'autoaway_message': [ opt_str, _('Away as a result of being idle') ],
@@ -69,25 +73,31 @@ class Config:
 		'last_status_msg_invisible': [ opt_str, '' ],
 		'last_status_msg_offline': [ opt_str, '' ],
 		'trayicon': [ opt_bool, True, '', True ],
-		'iconset': [ opt_str, 'dcraven', '', True ],
+		'iconset': [ opt_str, DEFAULT_ICONSET, '', True ],
 		'use_transports_iconsets': [ opt_bool, True, '', True ],
 		'inmsgcolor': [ opt_color, '#a34526', '', True ],
 		'outmsgcolor': [ opt_color, '#164e6f', '', True ],
 		'statusmsgcolor': [ opt_color, '#1eaa1e', '', True ],
 		'markedmsgcolor': [ opt_color, '#ff8080', '', True ],
 		'urlmsgcolor': [ opt_color, '#0000ff', '', True ],
-		'collapsed_rows': [ opt_str, '', '', True ],
+		'collapsed_rows': [ opt_str, '', _('List (space separated) of rows (accounts and groups) that are collapsed.'), True ],
 		'roster_theme': [ opt_str, 'gtk+', '', True ],
 		'saveposition': [ opt_bool, True ],
 		'mergeaccounts': [ opt_bool, False, '', True ],
 		'sort_by_show': [ opt_bool, True, '', True ],
+		'enable_zeroconf': [opt_bool, False, _('Enable link-local/zeroconf messaging')],
 		'use_speller': [ opt_bool, False, ],
-		'print_time': [ opt_str, 'always' ],
+		'ignore_incoming_xhtml': [ opt_bool, False, ],
+		'speller_language': [ opt_str, '', _('Language used by speller')],
+		'print_time': [ opt_str, 'always',  _('\'always\' - print time for every message.\n\'sometimes\' - print time every print_ichat_every_foo_minutes minute.\n\'never\' - never print time.')],
+		'print_time_fuzzy': [ opt_int, 0, _('Print time in chats using Fuzzy Clock. Value of fuzziness from 1 to 4, or 0 to disable fuzzyclock. 1 is the most precise clock, 4 the least precise one. This is used only if print_time is \'sometimes\'.') ],
 		'emoticons_theme': [opt_str, 'static', '', True ],
 		'ascii_formatting': [ opt_bool, True,
 			_('Treat * / _ pairs as possible formatting characters.'), True],
-		'show_ascii_formatting_chars': [ opt_bool, False , _('If True, do not '
+		'show_ascii_formatting_chars': [ opt_bool, True , _('If True, do not '
 			'remove */_ . So *abc* will be bold but with * * not removed.')],
+		'rst_formatting_outgoing_messages': [ opt_bool, False,
+			_('Uses ReStructured text markup to send HTML, plus ascii formatting if selected. For syntax, see http://docutils.sourceforge.net/docs/ref/rst/restructuredtext.html (If you want to use this, install docutils)')],
 		'sounds_on': [ opt_bool, True ],
 		# 'aplay', 'play', 'esdplay', 'artsplay' detected first time only
 		'soundplayer': [ opt_str, '' ],
@@ -95,12 +105,12 @@ class Config:
 		'custombrowser': [ opt_str, 'firefox' ],
 		'custommailapp': [ opt_str, 'mozilla-thunderbird -compose' ],
 		'custom_file_manager': [ opt_str, 'xffm' ],
-		'gc-hpaned-position': [opt_int, 540],
-		'gc_refer_to_nick_char': [opt_str, ','],
-		'gc_proposed_nick_char': [opt_str, '_'],
+		'gc-hpaned-position': [opt_int, 430],
+		'gc_refer_to_nick_char': [opt_str, ',', _('Character to add after nickname when using nick completion (tab) in group chat.')],
+		'gc_proposed_nick_char': [opt_str, '_', _('Character to propose to add after desired nickname when desired nickname is used by someone else in group chat.')],
 		'msgwin-x-position': [opt_int, -1], # Default is to let the window manager decide
 		'msgwin-y-position': [opt_int, -1], # Default is to let the window manager decide
-		'msgwin-width': [opt_int, 480],
+		'msgwin-width': [opt_int, 500],
 		'msgwin-height': [opt_int, 440],
 		'chat-msgwin-x-position': [opt_int, -1], # Default is to let the window manager decide
 		'chat-msgwin-y-position': [opt_int, -1], # Default is to let the window manager decide
@@ -108,7 +118,7 @@ class Config:
 		'chat-msgwin-height': [opt_int, 440],
 		'gc-msgwin-x-position': [opt_int, -1], # Default is to let the window manager decide
 		'gc-msgwin-y-position': [opt_int, -1], # Default is to let the window manager decide
-		'gc-msgwin-width': [opt_int, 480],
+		'gc-msgwin-width': [opt_int, 600],
 		'gc-msgwin-height': [opt_int, 440],
 		'single-msg-x-position': [opt_int, 0],
 		'single-msg-y-position': [opt_int, 0],
@@ -120,12 +130,13 @@ class Config:
 		'roster_height': [ opt_int, 400 ],
 		'latest_disco_addresses': [ opt_str, '' ],
 		'recently_groupchat': [ opt_str, '' ],
-		'before_time': [ opt_str, '[' ],
-		'after_time': [ opt_str, ']' ],
-		'before_nickname': [ opt_str, '' ],
-		'after_nickname': [ opt_str, ':' ],
+		'time_stamp': [ opt_str, '[%X] ', _('This option let you customize timestamp that is printed in conversation. For exemple "[%H:%M] " will show "[hour:minute] ". See python doc on strftime for full documentation: http://docs.python.org/lib/module-time.html') ],
+		'before_nickname': [ opt_str, '', _('Characters that are printed before the nickname in conversations') ],
+		'after_nickname': [ opt_str, ':', _('Characters that are printed after the nickname in conversations') ],
 		'send_os_info': [ opt_bool, True ],
+		'set_status_msg_from_current_music_track': [ opt_bool, False ],
 		'notify_on_new_gmail_email': [ opt_bool, True ],
+		'notify_on_new_gmail_email_extra': [ opt_bool, False ],
 		'usegpg': [ opt_bool, False, '', True ],
 		'use_gpg_agent': [ opt_bool, False ],
 		'change_roster_title': [ opt_bool, True, _('Add * and [n] in roster title?')],
@@ -134,32 +145,34 @@ class Config:
 		'send_on_ctrl_enter': [opt_bool, False, _('Send message on Ctrl+Enter and with Enter make new line (Mirabilis ICQ Client default behaviour).')],
 		'show_roster_on_startup': [opt_bool, True],
 		'key_up_lines': [opt_int, 25, _('How many lines to store for Ctrl+KeyUP.')],
-		'version': [ opt_str, '0.10.1' ], # which version created the config
+		'version': [ opt_str, defs.version ], # which version created the config
 		'search_engine': [opt_str, 'http://www.google.com/search?&q=%s&sourceid=gajim'],
 		'dictionary_url': [opt_str, 'WIKTIONARY', _("Either custom url with %s in it where %s is the word/phrase or 'WIKTIONARY' which means use wiktionary.")],
 		'always_english_wikipedia': [opt_bool, False],
 		'always_english_wiktionary': [opt_bool, True],
 		'remote_control': [opt_bool, True, _('If checked, Gajim can be controlled remotely using gajim-remote.'), True],
-		'chat_state_notifications': [opt_str, 'all'], # 'all', 'composing_only', 'disabled'
+		'networkmanager_support': [opt_bool, True, _('If True, listen to D-Bus signals from NetworkManager and change the status of accounts (provided they do not have listen_to_network_manager set to False and they sync with global status) based upon the status of the network connection.'), True],
+		'outgoing_chat_state_notifications': [opt_str, 'all', _('Sent chat state notifications. Can be one of all, composing_only, disabled.')],
+		'displayed_chat_state_notifications': [opt_str, 'all', _('Displayed chat state notifications in chat windows. Can be one of all, composing_only, disabled.')],
 		'autodetect_browser_mailer': [opt_bool, False, '', True],
-		'print_ichat_every_foo_minutes': [opt_int, 5],
+		'print_ichat_every_foo_minutes': [opt_int, 5, _('When not printing time for every message (print_time==sometimes), print it every x minutes.')],
 		'confirm_close_muc': [opt_bool, True, _('Ask before closing a group chat tab/window.')],
-		'confirm_close_muc_rooms': [opt_str, '', _('Always ask before closing group chat tab/window in this space separated list of room jids.')],
-		'noconfirm_close_muc_rooms': [opt_str, '', _('Never ask before closing group chat tab/window in this space separated list of room jids.')],
+		'confirm_close_muc_rooms': [opt_str, '', _('Always ask before closing group chat tab/window in this space separated list of group chat jids.')],
+		'noconfirm_close_muc_rooms': [opt_str, '', _('Never ask before closing group chat tab/window in this space separated list of group chat jids.')],
 		'notify_on_file_complete': [opt_bool, True],
 		'file_transfers_port': [opt_int, 28011],
 		'ft_override_host_to_send': [opt_str, '', _('Overrides the host we send for File Transfer in case of address translation/port forwarding.')], 
 		'conversation_font': [opt_str, ''],
 		'use_kib_mib': [opt_bool, False, _('IEC standard says KiB = 1024 bytes, KB = 1000 bytes.')],
 		'notify_on_all_muc_messages': [opt_bool, False],
-		'trayicon_notification_on_new_messages': [opt_bool, True],
+		'trayicon_notification_on_events': [opt_bool, True, _('Notify of events in the system trayicon.')],
 		'last_save_dir': [opt_str, ''],
 		'last_send_dir': [opt_str, ''],
 		'last_emoticons_dir': [opt_str, ''],
 		'last_sounds_dir': [opt_str, ''],
 		'tabs_position': [opt_str, 'top'],
-		'tabs_always_visible': [opt_bool, False, _('Show tab when only one conversation?'), '', True],
-		'tabs_border': [opt_bool, False, _('Show tab border if one conversation?')],
+		'tabs_always_visible': [opt_bool, False, _('Show tab when only one conversation?')],
+		'tabs_border': [opt_bool, False, _('Show tabbed notebook border in chat windows?')],
 		'tabs_close_button': [opt_bool, True, _('Show close button in tab?')],
 		'chat_avatar_width': [opt_int, 52],
 		'chat_avatar_height': [opt_int, 52],
@@ -173,31 +186,44 @@ class Config:
 		'notification_position_y': [opt_int, -1],
 		'notification_avatar_width': [opt_int, 48],
 		'notification_avatar_height': [opt_int, 48],
-		'muc_highlight_words': [opt_str, '', _('A semicolon-separated list of words that will be highlighted in multi-user chat.')],
+		'muc_highlight_words': [opt_str, '', _('A semicolon-separated list of words that will be highlighted in group chats.')],
 		'quit_on_roster_x_button': [opt_bool, False, _('If True, quits Gajim when X button of Window Manager is clicked. This setting is taken into account only if trayicon is used.')],
-		'set_xmpp://_handler_everytime': [opt_bool, False, _('If True, Gajim registers for xmpp:// on each startup.')],
+		'check_if_gajim_is_default': [opt_bool, True, _('If True, Gajim will check if it\'s the default jabber client on each startup.')],
 		'show_unread_tab_icon': [opt_bool, False, _('If True, Gajim will display an icon on each tab containing unread messages. Depending on the theme, this icon may be animated.')],
-		'show_status_msgs_in_roster': [opt_bool, True, _('If True, Gajim will display the status message, if not empty, for every contact under the contact name in roster window'), True],
+		'show_status_msgs_in_roster': [opt_bool, True, _('If True, Gajim will display the status message, if not empty, for every contact under the contact name in roster window.'), True],
 		'show_avatars_in_roster': [opt_bool, True, '', True],
 		'ask_avatars_on_startup': [opt_bool, True, _('If True, Gajim will ask for avatar each contact that did not have an avatar last time or has one cached that is too old.')],
-		#FIXME: remove you and make it Gajim will not; and/or his or *her* status messages
-		'print_status_in_chats': [opt_bool, True, _('If False, you will no longer see status line in chats when a contact changes his or her status and/or his status message.')],
+		'print_status_in_chats': [opt_bool, True, _('If False, Gajim will no longer print status line in chats when a contact changes his or her status and/or his or her status message.')],
+		'print_status_in_muc': [opt_str, 'in_and_out', _('can be "none", "all" or "in_and_out". If "none", Gajim will no longer print status line in groupchats when a member changes his or her status and/or his or her status message. If "all" Gajim will print all status messages. If "in_and_out", Gajim will only print FOO enters/leaves group chat.')],
 		'log_contact_status_changes': [opt_bool, False],
+		'just_connected_bg_color': [opt_str, '#adc3c6', _('Background color of contacts when they just signed in.')],
+		'just_disconnected_bg_color': [opt_str, '#ab6161', _('Background color of contacts when they just signed out.')],
 		'restored_messages_color': [opt_str, 'grey'],
-		'hide_avatar_of_transport': [opt_bool, False],
-		'roster_window_skip_taskbar': [opt_bool, False],
+		'restored_messages_small': [opt_bool, True, _('If True, restored messages will use a smaller font than the default one.')],
+		'hide_avatar_of_transport': [opt_bool, False, _('Don\'t show avatar for the transport itself.')],
+		'roster_window_skip_taskbar': [opt_bool, False, _('Don\'t show roster in the system taskbar.')],
 		'use_urgency_hint': [opt_bool, True, _('If True and installed GTK+ and PyGTK versions are at least 2.8, make the window flash (the default behaviour in most Window Managers) when holding pending events.')],
 		'notification_timeout': [opt_int, 5],
-		'send_sha_in_gc_presence': [opt_bool, True, _('Jabberd1.4 does not like sha info when one join a password protected room. Turn this option to False to stop sending sha info in groupchat presences')],
+		'send_sha_in_gc_presence': [opt_bool, True, _('Jabberd1.4 does not like sha info when one join a password protected group chat. Turn this option to False to stop sending sha info in group chat presences.')],
 		'one_message_window': [opt_str, 'always',
-			_('Controls the window where new messages are placed.\n\'always\' - All messages are sent to a single window.\n\'never\' - All messages get their own window.\n\'peracct\' - Messages for each account are sent to a specific window.\n\'pertype\' - Each message type (e.g., chats vs. groupchats) are sent to a specific window. Note, changing this option requires restarting Gajim before the changes will take effect')],
-		'show_avatar_in_chat': [opt_bool, True, _('If False, you will no longer see the avatar in the chat window')],
-		'escape_key_closes': [opt_bool, True, _('If True, pressing the escape key closes a tab/window')],
-		'always_hide_groupchat_buttons': [opt_bool, False, _('Hides the buttons in group chat window')],
-		'always_hide_chat_buttons': [opt_bool, False, _('Hides the buttons in two persons chat window')],
+#always, never, peracct, pertype should not be translated
+			_('Controls the window where new messages are placed.\n\'always\' - All messages are sent to a single window.\n\'never\' - All messages get their own window.\n\'peracct\' - Messages for each account are sent to a specific window.\n\'pertype\' - Each message type (e.g., chats vs. groupchats) are sent to a specific window. Note, changing this option requires restarting Gajim before the changes will take effect.')],
+		'show_avatar_in_chat': [opt_bool, True, _('If False, you will no longer see the avatar in the chat window.')],
+		'escape_key_closes': [opt_bool, True, _('If True, pressing the escape key closes a tab/window.')],
+		'always_hide_groupchat_buttons': [opt_bool, False, _('Hides the buttons in group chat window.')],
+		'always_hide_chat_buttons': [opt_bool, False, _('Hides the buttons in two persons chat window.')],
 		'hide_groupchat_banner': [opt_bool, False, _('Hides the banner in a group chat window')],
 		'hide_chat_banner': [opt_bool, False, _('Hides the banner in two persons chat window')],
-		'hide_groupchat_occupants_list': [opt_bool, False, _('Hides the room occupants list in groupchat window')],
+		'hide_groupchat_occupants_list': [opt_bool, False, _('Hides the group chat occupants list in group chat window.')],
+		'chat_merge_consecutive_nickname': [opt_bool, False, _('In a chat, show the nickname at the beginning of a line only when it\'s not the same person talking than in previous message.')],
+		'chat_merge_consecutive_nickname_indent': [opt_str, '  ', _('Indentation when using merge consecutive nickname.')],
+		'gc_nicknames_colors': [ opt_str, '#a34526:#c000ff:#0012ff:#388a99:#045723:#7c7c7c:#ff8a00:#94452d:#244b5a:#32645a', _('List of colors that will be used to color nicknames in group chats.'), True ],
+		'ctrl_tab_go_to_next_composing': [opt_bool, True, _('Ctrl-Tab go to next composing tab when none is unread.')],
+		'confirm_metacontacts': [ opt_str, '', _('Should we show the confirm metacontacts creation dialog or not? Empty string means we never show the dialog.')],
+		'enable_negative_priority': [ opt_bool, False, _('If True, you will be able to set a negative priority to your account in account modification window. BE CAREFUL, when you are logged in with a negative priority, you will NOT receive any message from your server.')],
+		'use_gnomekeyring': [opt_bool, True, _('If True, Gajim will use Gnome Keyring (if available) to store account passwords.')],
+		'show_contacts_number': [opt_bool, True, _('If True, Gajim will show number of online and total contacts in account and group rows.')],
+		'treat_incoming_messages': [ opt_str, '', _('Can be empty, \'chat\' or \'normal\'. If not empty, treat all incoming messages as if they were of this type')],
 	}
 
 	__options_per_key = {
@@ -208,6 +234,13 @@ class Config:
 			'password': [ opt_str, '' ],
 			'resource': [ opt_str, 'gajim', '', True ],
 			'priority': [ opt_int, 5, '', True ],
+			'adjust_priority_with_status': [ opt_bool, True, _('Priority will change automatically according to your status. Priorities are defined in autopriority_* options.') ],
+			'autopriority_online': [ opt_int, 50],
+			'autopriority_chat': [ opt_int, 50],
+			'autopriority_away': [ opt_int, 40],
+			'autopriority_xa': [ opt_int, 30],
+			'autopriority_dnd': [ opt_int, 20],
+			'autopriority_invisible': [ opt_int, 10],
 			'autoconnect': [ opt_bool, False, '', True ],
 			'autoreconnect': [ opt_bool, True ],
 			'active': [ opt_bool, True],
@@ -239,10 +272,20 @@ class Config:
 			'msgwin-y-position': [opt_int, -1], # Default is to let the wm decide
 			'msgwin-width': [opt_int, 480],
 			'msgwin-height': [opt_int, 440],
+			'listen_to_network_manager' : [opt_bool, True],
+			'is_zeroconf': [opt_bool, False],
+			'zeroconf_first_name': [ opt_str, '', '', True ],
+			'zeroconf_last_name': [ opt_str, '', '', True ],
+			'zeroconf_jabber_id': [ opt_str, '', '', True ],
+			'zeroconf_email': [ opt_str, '', '', True ],
 		}, {}),
 		'statusmsg': ({
 			'message': [ opt_str, '' ],
 		}, {}),
+		'defaultstatusmsg': ({
+			'enabled': [ opt_bool, False ],
+			'message': [ opt_str, '' ],
+		}, {}),
 		'soundevents': ({
 			'enabled': [ opt_bool, True ],
 			'path': [ opt_str, '' ],
@@ -273,8 +316,6 @@ class Config:
 			'bannerfontattrs': [ opt_str, 'B', '', True ],
 				
 			# http://www.pitt.edu/~nisg/cis/web/cgi/rgb.html
-			# FIXME: not black but the default color from gtk+ theme
-			'state_active_color': [ opt_color, 'black' ],
 			'state_inactive_color': [ opt_color, 'grey62' ],
 			'state_composing_color': [ opt_color, 'green4' ],
 			'state_paused_color': [ opt_color, 'mediumblue' ],
@@ -285,7 +326,27 @@ class Config:
 			'state_muc_directed_msg_color': [ opt_color, 'red2' ],
 		}, {}),
 		'contacts': ({
-			'gpg_enabled': [ opt_bool, True ],
+			'gpg_enabled': [ opt_bool, True, _('Is OpenPGP enabled for this contact?')],
+			'speller_language': [ opt_str, '', _('Language for which we want to check misspelled words')],
+		}, {}),
+		'rooms': ({
+			'speller_language': [ opt_str, '', _('Language for which we want to check misspelled words')],
+		}, {}),
+		'notifications': ({
+			'event': [opt_str, ''],
+			'recipient_type': [opt_str, 'all'],
+			'recipients': [opt_str, ''],
+			'status': [opt_str, 'all', _('all or space separated status')],
+			'tab_opened': [opt_str, 'both', _("'yes', 'no', or 'both'")],
+			'sound': [opt_str, '', _("'yes', 'no' or ''")],
+			'sound_file': [opt_str, ''],
+			'popup': [opt_str, '', _("'yes', 'no' or ''")],
+			'auto_open': [opt_str, '', _("'yes', 'no' or ''")],
+			'run_command': [opt_bool, False],
+			'command': [opt_str, ''],
+			'systray': [opt_str, '', _("'yes', 'no' or ''")],
+			'roster': [opt_str, '', _("'yes', 'no' or ''")],
+			'urgency_hint': [opt_bool, False],
 		}, {}),
 	}
 
@@ -296,7 +357,17 @@ class Config:
 		_('Movie'): _("I'm watching a movie."),
 		_('Working'): _("I'm working."),
 		_('Phone'): _("I'm on the phone."),
-		_('Out'): _("I'm out enjoying life"),
+		_('Out'): _("I'm out enjoying life."),
+	}
+
+	defaultstatusmsg_default = {
+		'online': [ False, _("I'm available.") ],
+		'chat': [ False, _("I'm free for chat.") ],
+		'away': [ False, _('Be right back.') ],
+		'xa': [ False, _("I'm not available.") ],
+		'dnd': [ False, _('Do not disturb.') ],
+		'invisible': [ False, _('Bye!') ],
+		'offline': [ False, _('Bye!') ],
 	}
 
 	soundevents_default = {
@@ -305,8 +376,9 @@ class Config:
 		'contact_connected': [ True, '../data/sounds/connected.wav' ],
 		'contact_disconnected': [ True, '../data/sounds/disconnected.wav' ],
 		'message_sent': [ True, '../data/sounds/sent.wav' ],
-		'muc_message_highlight': [ True, '../data/sounds/gc_message1.wav', _('Sound to play when a MUC message contains one of the words in muc_highlight_words, or when a MUC message contains your nickname.')],
-		'muc_message_received': [ True, '../data/sounds/gc_message2.wav', _('Sound to play when any MUC message arrives. (This setting is taken into account only if notify_on_all_muc_messages is True)') ],
+		'muc_message_highlight': [ True, '../data/sounds/gc_message1.wav', _('Sound to play when a group chat message contains one of the words in muc_highlight_words, or when a group chat message contains your nickname.')],
+		'muc_message_received': [ False, '../data/sounds/gc_message2.wav', _('Sound to play when any MUC message arrives.') ],
+		'gmail_received': [ False, '../data/sounds/message1.wav' ],
 	}
 
 	themes_default = {
@@ -377,7 +449,7 @@ class Config:
 		elif type[0] == 'string':
 			return self.is_valid_string(val)
 		else:
-			if sre.match(type[1], val):
+			if re.match(type[1], val):
 				return val
 			else:
 				return None
@@ -512,3 +584,8 @@ class Config:
 			self.set_per('soundevents', event, 'enabled', default[0])
 			self.set_per('soundevents', event, 'path', default[1])
 
+		for status in self.defaultstatusmsg_default:
+			default = self.defaultstatusmsg_default[status]
+			self.add_per('defaultstatusmsg', status)
+			self.set_per('defaultstatusmsg', status, 'enabled', default[0])
+			self.set_per('defaultstatusmsg', status, 'message', default[1])
