@@ -1,19 +1,11 @@
 ##	common/connection.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <nkour@jabber.org>
-##	- Dimitur Kirov <dkirov@gmail.com>
-##	- Travis Shirk <travis@pobox.com>
 ##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2003-2004 Vincent Hanquez <tab@snarc.org>
+## Copyright (C) 2003-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2005-2006 Dimitur Kirov <dkirov@gmail.com>
+## Copyright (C) 2005-2006 Travis Shirk <travis@pobox.com>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -28,7 +20,12 @@
 
 import os
 import random
-random.seed()
+
+try:
+	randomsource = random.SystemRandom()
+except:
+	randomsource = random.Random()
+	randomsource.seed()
 
 import signal
 if os.name != 'nt':
@@ -38,12 +35,12 @@ import common.xmpp
 from common import helpers
 from common import gajim
 from common import GnuPG
+from common import passwords
 
 from connection_handlers import *
 USE_GPG = GnuPG.USE_GPG
 
-from common import i18n
-_ = i18n._
+from common.rst_xhtml_generator import create_xhtml
 
 class Connection(ConnectionHandlers):
 	'''Connection class'''
@@ -54,21 +51,25 @@ class Connection(ConnectionHandlers):
 		self.connection = None # xmpppy ClientCommon instance
 		# this property is used to prevent double connections
 		self.last_connection = None # last ClientCommon instance
+		self.is_zeroconf = False
 		self.gpg = None
 		self.status = ''
+		self.priority = gajim.get_priority(name, 'offline')
 		self.old_show = ''
 		# increase/decrease default timeout for server responses
 		self.try_connecting_for_foo_secs = 45
 		# holds the actual hostname to which we are connected
 		self.connected_hostname = None
 		self.time_to_reconnect = None
+		self.last_time_to_reconnect = None
 		self.new_account_info = None
 		self.bookmarks = []
+		self.annotations = {}
 		self.on_purpose = False
 		self.last_io = gajim.idlequeue.current_time()
 		self.last_sent = []
 		self.last_history_line = {}
-		self.password = gajim.config.get_per('accounts', name, 'password')
+		self.password = passwords.get_password(name)
 		self.server_resource = gajim.config.get_per('accounts', name, 'resource')
 		if gajim.config.get_per('accounts', self.name, 'keep_alives_enabled'):
 			self.keepalives = gajim.config.get_per('accounts', self.name,'keep_alive_every_foo_secs')
@@ -78,7 +79,7 @@ class Connection(ConnectionHandlers):
 		# Do we continue connection when we get roster (send presence,get vcard...)
 		self.continue_connect_info = None
 		if USE_GPG:
-			self.gpg = GnuPG.GnuPG()
+			self.gpg = GnuPG.GnuPG(gajim.config.get('use_gpg_agent'))
 			gajim.config.set('usegpg', True)
 		else:
 			gajim.config.set('usegpg', False)
@@ -87,8 +88,13 @@ class Connection(ConnectionHandlers):
 		self.on_connect_failure = None
 		self.retrycount = 0
 		self.jids_for_auto_auth = [] # list of jid to auto-authorize
-		
+		self.muc_jid = {} # jid of muc server for each transport type
+		self.available_transports = {} # list of available transports on this
+		# server {'icq': ['icq.server.com', 'icq2.server.com'], }
+		self.vcard_supported = True
+		self.metacontacts_supported = True
 	# END __init__
+
 	def put_event(self, ev):
 		if gajim.handlers.has_key(ev[0]):
 			gajim.handlers[ev[0]](self.name, ev[1])
@@ -101,7 +107,7 @@ class Connection(ConnectionHandlers):
 	def _reconnect(self):
 		# Do not try to reco while we are already trying
 		self.time_to_reconnect = None
-		if self.connected < 2: #connection failed
+		if self.connected < 2: # connection failed
 			gajim.log.debug('reconnect')
 			self.retrycount += 1
 			signed = self.get_signed_msg(self.status)
@@ -117,40 +123,50 @@ class Connection(ConnectionHandlers):
 		self.on_purpose = on_purpose
 		self.connected = 0
 		self.time_to_reconnect = None
+		self.privacy_rules_supported = False
 		if self.connection:
 			# make sure previous connection is completely closed
 			gajim.proxy65_manager.disconnect(self.connection)
 			self.connection.disconnect()
+			self.last_connection = None
 			self.connection = None
 	
 	def _disconnectedReconnCB(self):
 		'''Called when we are disconnected'''
 		gajim.log.debug('disconnectedReconnCB')
-		if self.connected > 1:
-			# we cannot change our status to offline or connectiong
+		if gajim.account_is_connected(self.name):
+			# we cannot change our status to offline or connecting
 			# after we auth to server
 			self.old_show = STATUS_LIST[self.connected]
 		self.connected = 0
 		self.dispatch('STATUS', 'offline')
 		if not self.on_purpose:
 			self.disconnect()
-			if gajim.config.get_per('accounts', self.name, 'autoreconnect') \
-				and self.retrycount <= 10:
+			if gajim.config.get_per('accounts', self.name, 'autoreconnect'):
 				self.connected = 1
 				self.dispatch('STATUS', 'connecting')
-				self.time_to_reconnect = 10
+				if gajim.status_before_autoaway[self.name]:
+					# We were auto away. So go back online
+					self.status = gajim.status_before_autoaway[self.name]
+					gajim.status_before_autoaway[self.name] = ''
+					self.old_show = 'online'
 				# this check has moved from _reconnect method
-				if self.retrycount > 5:
-					self.time_to_reconnect = 20
-				else:
-					self.time_to_reconnect = 10
-				gajim.idlequeue.set_alarm(self._reconnect_alarm, 
-										self.time_to_reconnect)
+				# do exponential backoff until 15 minutes,
+				# then small linear increase
+				if self.retrycount < 2 or self.last_time_to_reconnect is None:
+					self.last_time_to_reconnect = 5
+				if self.last_time_to_reconnect < 800:
+					self.last_time_to_reconnect *= 1.5
+				self.last_time_to_reconnect += randomsource.randint(0, 5)
+				self.time_to_reconnect = int(self.last_time_to_reconnect)
+				gajim.log.debug("Reconnect to %s in %ss", self.name, self.time_to_reconnect)
+				gajim.idlequeue.set_alarm(self._reconnect_alarm,
+					self.time_to_reconnect)
 			elif self.on_connect_failure:
 				self.on_connect_failure()
 				self.on_connect_failure = None
 			else:
-					# show error dialog
+				# show error dialog
 				self._connection_lost()
 		else:
 			self.disconnect()
@@ -158,22 +174,23 @@ class Connection(ConnectionHandlers):
 	# END disconenctedReconnCB
 	
 	def _connection_lost(self):
-		self.dispatch('ERROR',
-		(_('Connection with account "%s" has been lost') % self.name,
-		_('To continue sending and receiving messages, you will need to reconnect.')))
+		self.disconnect(on_purpose = False)
+		self.dispatch('STATUS', 'offline')
+		self.dispatch('CONNECTION_LOST',
+			(_('Connection with account "%s" has been lost') % self.name,
+			_('Reconnect manually.')))
 
 	def _event_dispatcher(self, realm, event, data):
 		if realm == common.xmpp.NS_REGISTER:
 			if event == common.xmpp.features_nb.REGISTER_DATA_RECEIVED:
-				# data is (agent, DataFrom, is_form)
-				if self.new_account_info and\
+				# data is (agent, DataFrom, is_form, error_msg)
+				if self.new_account_info and \
 				self.new_account_info['hostname'] == data[0]:
-					#it's a new account
+					# it's a new account
 					if not data[1]: # wrong answer
-						print self.connection.lastErr
 						self.dispatch('ACC_NOT_OK', (
-							_('Transport %s answered wrongly to register request.') % \
-							data[0]))
+							_('Transport %s answered wrongly to register request: %s')\
+							% (data[0], data[3])))
 						return
 					req = data[1].asDict()
 					req['username'] = self.new_account_info['name']
@@ -182,24 +199,25 @@ class Connection(ConnectionHandlers):
 						if not common.xmpp.isResultNode(result):
 							self.dispatch('ACC_NOT_OK', (result.getError()))
 							return
-						self.connected = 0
 						self.password = self.new_account_info['password']
 						if USE_GPG:
-							self.gpg = GnuPG.GnuPG()
+							self.gpg = GnuPG.GnuPG(gajim.config.get('use_gpg_agent'))
 							gajim.config.set('usegpg', True)
 						else:
 							gajim.config.set('usegpg', False)
 						gajim.connections[self.name] = self
 						self.dispatch('ACC_OK', (self.new_account_info))
 						self.new_account_info = None
-						self.connection = None
+						if self.connection:
+							self.connection.UnregisterDisconnectHandler(self._on_new_account)
+						self.disconnect(on_purpose=True)
 					common.xmpp.features_nb.register(self.connection, data[0],
 						req, _on_register_result)
 					return
 				if not data[1]: # wrong answer
 					self.dispatch('ERROR', (_('Invalid answer'),
-						_('Transport %s answered wrongly to register request.') % \
-						data[0]))
+						_('Transport %s answered wrongly to register request: %s') % \
+						(data[0], data[3])))
 					return
 				is_form = data[2]
 				if is_form:
@@ -207,6 +225,34 @@ class Connection(ConnectionHandlers):
 				else:
 					conf = data[1].asDict()
 				self.dispatch('REGISTER_AGENT_INFO', (data[0], conf, is_form))
+		elif realm == common.xmpp.NS_PRIVACY:
+			if event == common.xmpp.features_nb.PRIVACY_LISTS_RECEIVED:
+				# data is (list)
+				self.dispatch('PRIVACY_LISTS_RECEIVED', (data))
+			elif event == common.xmpp.features_nb.PRIVACY_LIST_RECEIVED:
+				# data is (resp)
+				if not data:
+					return
+				rules = []
+				name = data.getTag('query').getTag('list').getAttr('name')
+				for child in data.getTag('query').getTag('list').getChildren():
+					dict_item = child.getAttrs()
+					childs = []
+					if dict_item.has_key('type'):
+						for scnd_child in child.getChildren():
+							childs += [scnd_child.getName()]
+						rules.append({'action':dict_item['action'],
+							'type':dict_item['type'], 'order':dict_item['order'],
+							'value':dict_item['value'], 'child':childs})
+					else:
+						for scnd_child in child.getChildren():
+							childs.append(scnd_child.getName())
+						rules.append({'action':dict_item['action'],
+							'order':dict_item['order'], 'child':childs})
+				self.dispatch('PRIVACY_LIST_RECEIVED', (name, rules))
+			elif event == common.xmpp.features_nb.PRIVACY_LISTS_ACTIVE_DEFAULT:
+				# data is (dict)
+				self.dispatch('PRIVACY_LISTS_ACTIVE_DEFAULT', (data))
 		elif realm == '':
 			if event == common.xmpp.transports.DATA_RECEIVED:
 				self.dispatch('STANZA_ARRIVED', unicode(data, errors = 'ignore'))
@@ -301,8 +347,10 @@ class Connection(ConnectionHandlers):
 		self._hosts = [ {'host': h, 'port': p, 'prio': 10, 'weight': 10} ]
 		self._hostname = hostname
 		if use_srv:
-			# add request for srv query to the resolve, on result '_on_resolve' will be called
-			gajim.resolver.resolve('_xmpp-client._tcp.' + h.encode('utf-8'), self._on_resolve)
+			# add request for srv query to the resolve, on result '_on_resolve'
+			# will be called
+			gajim.resolver.resolve('_xmpp-client._tcp.' + helpers.idn_to_ascii(h),
+				self._on_resolve)
 		else:
 			self._on_resolve('', [])
 
@@ -333,12 +381,16 @@ class Connection(ConnectionHandlers):
 			host = self.select_next_host(self._hosts)
 			self._current_host = host
 			self._hosts.remove(host)
+
+			# FIXME: this is a hack; need a better way
+			if self.on_connect_success == self._on_new_account:
+				con.RegisterDisconnectHandler(self._on_new_account)
+
 			con.connect((host['host'], host['port']), proxy = self._proxy,
 				secure = self._secure)
 			return
 		else:
-			if not retry or self.retrycount > 10:
-				self.retrycount = 0
+			if not retry and self.retrycount == 0:
 				self.time_to_reconnect = None
 				if self.on_connect_failure:
 					self.on_connect_failure()
@@ -356,7 +408,8 @@ class Connection(ConnectionHandlers):
 			if not self.retrycount and self.connected != 0:
 				self.disconnect(on_purpose = True)
 				self.dispatch('STATUS', 'offline')
-				self.dispatch('ERROR', (_('Could not connect to "%s"') % self._hostname,
+				self.dispatch('CONNECTION_LOST',
+					(_('Could not connect to "%s"') % self._hostname,
 					_('Check your connection or try again later.')))
 
 	def _connect_success(self, con, con_type):
@@ -372,8 +425,6 @@ class Connection(ConnectionHandlers):
 		con.RegisterDisconnectHandler(self._disconnectedReconnCB)
 		gajim.log.debug(_('Connected to server %s:%s with %s') % (self._current_host['host'],
 			self._current_host['port'], con_type))
-		# Ask metacontacts before roster
-		self.get_metacontacts()
 		self._register_handlers(con, con_type)
 		return True
 
@@ -392,7 +443,8 @@ class Connection(ConnectionHandlers):
 		if not con:
 			self.disconnect(on_purpose = True)
 			self.dispatch('STATUS', 'offline')
-			self.dispatch('ERROR', (_('Could not connect to "%s"') % self._hostname,
+			self.dispatch('CONNECTION_LOST',
+				(_('Could not connect to "%s"') % self._hostname,
 				_('Check your connection or try again later')))
 			if self.on_connect_auth:
 				self.on_connect_auth(None)
@@ -408,6 +460,7 @@ class Connection(ConnectionHandlers):
 		if auth:
 			self.last_io = gajim.idlequeue.current_time()
 			self.connected = 2
+			self.retrycount = 0
 			if self.on_connect_auth:
 				self.on_connect_auth(con)
 				self.on_connect_auth = None
@@ -426,9 +479,53 @@ class Connection(ConnectionHandlers):
 	# END connect
 
 	def quit(self, kill_core):
-		if kill_core and self.connected > 1:
+		if kill_core and gajim.account_is_connected(self.name):
 			self.disconnect(on_purpose = True)
 	
+	def get_privacy_lists(self):
+		if not self.connection:
+			return
+		common.xmpp.features_nb.getPrivacyLists(self.connection)
+
+	def get_active_default_lists(self):
+		if not self.connection:
+			return
+		common.xmpp.features_nb.getActiveAndDefaultPrivacyLists(self.connection)
+	
+	def del_privacy_list(self, privacy_list):
+		if not self.connection:
+			return
+		def _on_del_privacy_list_result(result):
+			if result:
+				self.dispatch('PRIVACY_LIST_REMOVED', privacy_list)
+			else:
+				self.dispatch('ERROR', (_('Error while removing privacy list'),
+					_('Privacy list %s has not been removed. It is maybe active in '
+					'one of your connected resources. Desactivate it and try '
+					'again.') % privacy_list))
+		common.xmpp.features_nb.delPrivacyList(self.connection, privacy_list,
+			_on_del_privacy_list_result)
+	
+	def get_privacy_list(self, title):
+		if not self.connection:
+			return
+		common.xmpp.features_nb.getPrivacyList(self.connection, title)
+	
+	def set_privacy_list(self, listname, tags):
+		if not self.connection:
+			return
+		common.xmpp.features_nb.setPrivacyList(self.connection, listname, tags)
+	
+	def set_active_list(self, listname):
+		if not self.connection:
+			return
+		common.xmpp.features_nb.setActivePrivacyList(self.connection, listname, 'active')
+
+	def set_default_list(self, listname):
+		if not self.connection:
+			return
+		common.xmpp.features_nb.setDefaultPrivacyList(self.connection, listname)
+	
 	def build_privacy_rule(self, name, action):
 		'''Build a Privacy rule stanza for invisibility'''
 		iq = common.xmpp.Iq('set', common.xmpp.NS_PRIVACY, xmlns = '')
@@ -438,6 +535,8 @@ class Connection(ConnectionHandlers):
 		return iq
 
 	def activate_privacy_rule(self, name):
+		if not self.connection:
+			return
 		'''activate a privacy rule'''
 		iq = common.xmpp.Iq('set', common.xmpp.NS_PRIVACY, xmlns = '')
 		iq.getTag('query').setTag('active', {'name': name})
@@ -461,14 +560,15 @@ class Connection(ConnectionHandlers):
 			# active the privacy rule
 			self.privacy_rules_supported = True
 			self.activate_privacy_rule('invisible')
-		prio = unicode(gajim.config.get_per('accounts', self.name, 'priority'))
-		p = common.xmpp.Presence(typ = ptype, priority = prio, show = show)
+		priority = unicode(gajim.get_priority(self.name, show))
+		p = common.xmpp.Presence(typ = ptype, priority = priority, show = show)
 		p = self.add_sha(p, ptype != 'unavailable')
 		if msg:
 			p.setStatus(msg)
 		if signed:
 			p.setTag(common.xmpp.NS_SIGNED + ' x').setData(signed)
 		self.connection.send(p)
+		self.priority = priority
 		self.dispatch('STATUS', 'invisible')
 		if initial:
 			#ask our VCard
@@ -476,6 +576,9 @@ class Connection(ConnectionHandlers):
 
 			#Get bookmarks from private namespace
 			self.get_bookmarks()
+			
+			#Get annotations
+			self.get_annotations()
 
 			#Inform GUI we just signed in
 			self.dispatch('SIGNED_IN', ())
@@ -521,22 +624,19 @@ class Connection(ConnectionHandlers):
 		if self.connection:
 			con.set_send_timeout(self.keepalives, self.send_keepalive)
 			self.connection.onreceive(None)
-			# Ask metacontacts before roster
-			self.get_metacontacts()
+			iq = common.xmpp.Iq('get', common.xmpp.NS_PRIVACY, xmlns = '')
+			id = self.connection.getAnID()
+			iq.setID(id)
+			self.awaiting_answers[id] = (PRIVACY_ARRIVED, )
+			self.connection.send(iq)
 
-	def change_status(self, show, msg, sync = False, auto = False):
+	def change_status(self, show, msg, auto = False):
 		if not show in STATUS_LIST:
 			return -1
 		sshow = helpers.get_xmpp_show(show)
 		if not msg:
 			msg = ''
 		keyID = gajim.config.get_per('accounts', self.name, 'keyid')
-		if keyID and USE_GPG and not msg:
-			lowered_uf_status_msg = helpers.get_uf_show(show).lower()
-			# do not show I'm invisible!
-			if lowered_uf_status_msg == _('invisible'):
-				lowered_uf_status_msg = _('offline')
-			msg = _("I'm %s") % lowered_uf_status_msg
 		signed = ''
 		if not auto and not show == 'offline':
 			signed = self.get_signed_msg(msg)
@@ -576,9 +676,8 @@ class Connection(ConnectionHandlers):
 				iq = self.build_privacy_rule('visible', 'allow')
 				self.connection.send(iq)
 				self.activate_privacy_rule('visible')
-			prio = unicode(gajim.config.get_per('accounts', self.name,
-				'priority'))
-			p = common.xmpp.Presence(typ = None, priority = prio, show = sshow)
+			priority = unicode(gajim.get_priority(self.name, sshow))
+			p = common.xmpp.Presence(typ = None, priority = priority, show = sshow)
 			p = self.add_sha(p)
 			if msg:
 				p.setStatus(msg)
@@ -586,6 +685,7 @@ class Connection(ConnectionHandlers):
 				p.setTag(common.xmpp.NS_SIGNED + ' x').setData(signed)
 			if self.connection:
 				self.connection.send(p)
+				self.priority = priority
 			self.dispatch('STATUS', show)
 
 	def _on_disconnected(self):
@@ -596,18 +696,24 @@ class Connection(ConnectionHandlers):
 	def get_status(self):
 		return STATUS_LIST[self.connected]
 
-	def send_motd(self, jid, subject = '', msg = ''):
+
+	def send_motd(self, jid, subject = '', msg = '', xhtml = None):
 		if not self.connection:
 			return
-		msg_iq = common.xmpp.Message(to = jid, body = msg, subject = subject)
+		msg_iq = common.xmpp.Message(to = jid, body = msg, subject = subject,
+			xhtml = xhtml)
+
 		self.connection.send(msg_iq)
 
 	def send_message(self, jid, msg, keyID, type = 'chat', subject='',
-	chatstate = None, msg_id = None, composing_jep = None, resource = None):
+	chatstate = None, msg_id = None, composing_jep = None, resource = None,
+	user_nick = None, xhtml = None):
 		if not self.connection:
-			return
+			return 1
+		if msg and not xhtml and gajim.config.get('rst_formatting_outgoing_messages'):
+			xhtml = create_xhtml(msg)
 		if not msg and chatstate is None:
-			return
+			return 2
 		fjid = jid
 		if resource:
 			fjid += '/' + resource
@@ -615,35 +721,53 @@ class Connection(ConnectionHandlers):
 		msgenc = ''
 		if keyID and USE_GPG:
 			#encrypt
-			msgenc = self.gpg.encrypt(msg, [keyID])
-			if msgenc:
+			msgenc, error = self.gpg.encrypt(msg, [keyID])
+			if msgenc and not error:
 				msgtxt = '[This message is encrypted]'
 				lang = os.getenv('LANG')
-				if lang is not None or lang != 'en': # we're not english
-					msgtxt = _('[This message is encrypted]') +\
-						' ([This message is encrypted])' # one  in locale and one en
+				if lang is not None and lang != 'en': # we're not english
+					# one  in locale and one en
+					msgtxt = _('[This message is *encrypted* (See :JEP:`27`]') +\
+						' ([This message is *encrypted* (See :JEP:`27`])'
+			else:
+				# Encryption failed, do not send message
+				tim = time.localtime()
+				self.dispatch('MSGNOTSENT', (jid, error, msgtxt, tim))
+				return 3
+		if msgtxt and not xhtml and gajim.config.get(
+			'rst_formatting_outgoing_messages'):
+			# Generate a XHTML part using reStructured text markup
+			xhtml = create_xhtml(msgtxt)
 		if type == 'chat':
-			msg_iq = common.xmpp.Message(to = fjid, body = msgtxt, typ = type)
+			msg_iq = common.xmpp.Message(to = fjid, body = msgtxt, typ = type,
+				xhtml = xhtml)
 		else:
 			if subject:
 				msg_iq = common.xmpp.Message(to = fjid, body = msgtxt,
-					typ = 'normal', subject = subject)
+					typ = 'normal', subject = subject, xhtml = xhtml)
 			else:
 				msg_iq = common.xmpp.Message(to = fjid, body = msgtxt,
-					typ = 'normal')
+					typ = 'normal', xhtml = xhtml)
 		if msgenc:
 			msg_iq.setTag(common.xmpp.NS_ENCRYPTED + ' x').setData(msgenc)
 
+		# JEP-0172: user_nickname
+		if user_nick:
+			msg_iq.setTag('nick', namespace = common.xmpp.NS_NICK).setData(
+				user_nick)
+
 		# chatstates - if peer supports jep85 or jep22, send chatstates
 		# please note that the only valid tag inside a message containing a <body>
 		# tag is the active event
 		if chatstate is not None:
-			if composing_jep == 'JEP-0085' or not composing_jep:
+			if (composing_jep == 'JEP-0085' or not composing_jep) and \
+			composing_jep != 'asked_once':
 				# JEP-0085
 				msg_iq.setTag(chatstate, namespace = common.xmpp.NS_CHATSTATES)
-			if composing_jep == 'JEP-0022' or not composing_jep:
+			if composing_jep in ('JEP-0022', 'asked_once') or not composing_jep:
 				# JEP-0022
-				chatstate_node = msg_iq.setTag('x', namespace = common.xmpp.NS_EVENT)
+				chatstate_node = msg_iq.setTag('x',
+					namespace = common.xmpp.NS_EVENT)
 				if not msgtxt: # when no <body>, add <id>
 					if not msg_id: # avoid putting 'None' in <id> tag
 						msg_id = ''
@@ -653,7 +777,8 @@ class Connection(ConnectionHandlers):
 					chatstate_node.addChild(name = 'composing') 
 
 		self.connection.send(msg_iq)
-		no_log_for = gajim.config.get_per('accounts', self.name, 'no_log_for')
+		no_log_for = gajim.config.get_per('accounts', self.name, 'no_log_for')\
+			.split()
 		ji = gajim.get_jid_without_resource(jid)
 		if self.name not in no_log_for and ji not in no_log_for:
 			log_msg = msg
@@ -688,7 +813,7 @@ class Connection(ConnectionHandlers):
 		self.connection.send(p)
 
 	def request_subscription(self, jid, msg = '', name = '', groups = [],
-	auto_auth = False):
+	auto_auth = False, user_nick = ''):
 		if not self.connection:
 			return
 		gajim.log.debug('subscription request for %s' % jid)
@@ -706,10 +831,11 @@ class Connection(ConnectionHandlers):
 		self.connection.send(iq)
 
 		p = common.xmpp.Presence(jid, 'subscribe')
+		if user_nick:
+			p.setTag('nick', namespace = common.xmpp.NS_NICK).setData(user_nick)
 		p = self.add_sha(p)
-		if not msg:
-			msg = _('I would like to add you to my roster.')
-		p.setStatus(msg)
+		if msg:
+			p.setStatus(msg)
 		self.connection.send(p)
 
 	def send_authorization(self, jid):
@@ -755,7 +881,7 @@ class Connection(ConnectionHandlers):
 		if self.connection:
 			self.connection.getRoster().setItem(jid = jid, name = name,
 				groups = groups)
-	
+
 	def new_account(self, name, config, sync = False):
 		# If a connection already exist we cannot create a new account
 		if self.connection:
@@ -774,6 +900,8 @@ class Connection(ConnectionHandlers):
 			return
 		self.on_connect_failure = None
 		self.connection = con
+		#if con:
+		#	con.RegisterDisconnectHandler(self._on_new_account)
 		common.xmpp.features_nb.getRegInfo(con, self._hostname)
 
 	def account_changed(self, new_name):
@@ -792,6 +920,10 @@ class Connection(ConnectionHandlers):
 	def request_os_info(self, jid, resource):
 		if not self.connection:
 			return
+		# If we are invisible, do not request
+		if self.connected == gajim.SHOW_LIST.index('invisible'):
+			self.dispatch('OS_INFO', (jid, resource, _('Not fetched because of invisible status'), _('Not fetched because of invisible status')))
+			return
 		to_whom_jid = jid
 		if resource:
 			to_whom_jid += '/' + resource
@@ -833,12 +965,39 @@ class Connection(ConnectionHandlers):
 			# Only add optional elements if not empty
 			# Note: need to handle both None and '' as empty
 			#   thus shouldn't use "is not None"
-			if bm['nick']:
+			if bm.get('nick', None):
 				iq5 = iq4.setTagData('nick', bm['nick'])
-			if bm['password']:
+			if bm.get('password', None):
 				iq5 = iq4.setTagData('password', bm['password'])
+			if bm.get('print_status', None):
+				iq5 = iq4.setTagData('print_status', bm['print_status'])
 		self.connection.send(iq)
 
+	def get_annotations(self):
+		'''Get Annonations from storage as described in XEP 0048, and XEP 0145'''
+		self.annotations = {}
+		if not self.connection:
+			return
+		iq = common.xmpp.Iq(typ='get')
+		iq2 = iq.addChild(name='query', namespace='jabber:iq:private')
+		iq2.addChild(name='storage', namespace='storage:rosternotes')
+		self.connection.send(iq)
+
+	def store_annotations(self):
+		'''Set Annonations in private storage as described in XEP 0048, and XEP 0145'''
+		if not self.connection:
+			return
+		iq = common.xmpp.Iq(typ='set')
+		iq2 = iq.addChild(name='query', namespace='jabber:iq:private')
+		iq3 = iq2.addChild(name='storage', namespace='storage:rosternotes')
+		for jid in self.annotations.keys():
+			if self.annotations[jid]:
+				iq4 = iq3.addChild(name = "note")
+				iq4.setAttr('jid', jid)
+				iq4.setData(self.annotations[jid])
+		self.connection.send(iq)
+
+
 	def get_metacontacts(self):
 		'''Get metacontacts list from storage as described in JEP 0049'''
 		if not self.connection:
@@ -846,6 +1005,9 @@ class Connection(ConnectionHandlers):
 		iq = common.xmpp.Iq(typ='get')
 		iq2 = iq.addChild(name='query', namespace='jabber:iq:private')
 		iq2.addChild(name='storage', namespace='storage:metacontacts')
+		id = self.connection.getAnID()
+		iq.setID(id)
+		self.awaiting_answers[id] = (METACONTACTS_ARRIVED, )
 		self.connection.send(iq)
 
 	def store_metacontacts(self, tags_list):
@@ -867,18 +1029,20 @@ class Connection(ConnectionHandlers):
 	def send_agent_status(self, agent, ptype):
 		if not self.connection:
 			return
-		p = common.xmpp.Presence(to = agent, typ = ptype)
+		show = helpers.get_xmpp_show(STATUS_LIST[self.connected])
+		p = common.xmpp.Presence(to = agent, typ = ptype, show = show)
 		p = self.add_sha(p, ptype != 'unavailable')
 		self.connection.send(p)
 
-	def join_gc(self, nick, room, server, password):
+	def join_gc(self, nick, room_jid, password):
+		# FIXME: This room JID needs to be normalized; see #1364
 		if not self.connection:
 			return
 		show = helpers.get_xmpp_show(STATUS_LIST[self.connected])
 		if show == 'invisible':
 			# Never join a room when invisible
 			return
-		p = common.xmpp.Presence(to = '%s@%s/%s' % (room, server, nick),
+		p = common.xmpp.Presence(to = '%s/%s' % (room_jid, nick),
 			show = show, status = self.status)
 		if gajim.config.get('send_sha_in_gc_presence'):
 			p = self.add_sha(p)
@@ -886,18 +1050,22 @@ class Connection(ConnectionHandlers):
 		if password:
 			t.setTagData('password', password)
 		self.connection.send(p)
-		#last date/time in history to avoid duplicate
-		# FIXME: This JID needs to be normalized; see #1364
-		jid='%s@%s' % (room, server)
-		last_log = gajim.logger.get_last_date_that_has_logs(jid, is_room = True)
-		if last_log is None:
-			last_log = 0
-		self.last_history_line[jid]= last_log
 
-	def send_gc_message(self, jid, msg):
+		# last date/time in history to avoid duplicate
+		if not self.last_history_line.has_key(room_jid): 
+			# Not in memory, get it from DB
+			last_log = gajim.logger.get_last_date_that_has_logs(room_jid,
+				is_room = True)
+			if last_log is None:
+				last_log = 0
+			self.last_history_line[room_jid]= last_log
+
+	def send_gc_message(self, jid, msg, xhtml = None):
 		if not self.connection:
 			return
-		msg_iq = common.xmpp.Message(jid, msg, typ = 'groupchat')
+		if not xhtml and gajim.config.get('rst_formatting_outgoing_messages'):
+			xhtml = create_xhtml(msg)
+		msg_iq = common.xmpp.Message(jid, msg, typ = 'groupchat', xhtml = xhtml)
 		self.connection.send(msg_iq)
 		self.dispatch('MSGSENT', (jid, msg))
 
@@ -912,13 +1080,6 @@ class Connection(ConnectionHandlers):
 			to = room_jid)
 		self.connection.send(iq)
 
-	def change_gc_nick(self, room_jid, nick):
-		if not self.connection:
-			return
-		p = common.xmpp.Presence(to = '%s/%s' % (room_jid, nick))
-		p = self.add_sha(p)
-		self.connection.send(p)
-
 	def send_gc_status(self, nick, jid, show, status):
 		if not self.connection:
 			return
@@ -927,14 +1088,17 @@ class Connection(ConnectionHandlers):
 		ptype = None
 		if show == 'offline':
 			ptype = 'unavailable'
-		show = helpers.get_xmpp_show(show)
+		xmpp_show = helpers.get_xmpp_show(show)
 		p = common.xmpp.Presence(to = '%s/%s' % (jid, nick), typ = ptype,
-			show = show, status = status)
-		if gajim.config.get('send_sha_in_gc_presence'):
+			show = xmpp_show, status = status)
+		if gajim.config.get('send_sha_in_gc_presence') and show != 'offline':
 			p = self.add_sha(p, ptype != 'unavailable')
 		# send instantly so when we go offline, status is sent to gc before we
 		# disconnect from jabber server
 		self.connection.send(p)
+		# Save the time we quit to avoid duplicate logs AND be faster than 
+		# get that date from DB
+		self.last_history_line[jid] = time.time()
 
 	def gc_set_role(self, room_jid, nick, role, reason = ''):
 		'''role is for all the life of the room so it's based on nick'''
@@ -1023,11 +1187,11 @@ class Connection(ConnectionHandlers):
 		self.connection.send(iq)
 
 	def unregister_account(self, on_remove_success):
-		# no need to write this as a class method and keep the value of on_remove_success
-		# as a class property as pass it as an argument
+		# no need to write this as a class method and keep the value of
+		# on_remove_success as a class property as pass it as an argument
 		def _on_unregister_account_connect(con):
 			self.on_connect_auth = None
-			if self.connected > 1:
+			if gajim.account_is_connected(self.name):
 				hostname = gajim.config.get_per('accounts', self.name, 'hostname')
 				iq = common.xmpp.Iq(typ = 'set', to = hostname)
 				q = iq.setTag(common.xmpp.NS_REGISTER + ' query').setTag('remove')
