@@ -16,19 +16,23 @@
 ## GNU General Public License for more details.
 ##
 
-import sre
+import re
+import locale
 import os
+import subprocess
 import urllib
 import errno
 import select
-import sys
 import sha
 from encodings.punycode import punycode_encode
+from encodings import idna
 
 import gajim
-import i18n
+from i18n import Q_
+from i18n import ngettext
 from xmpp_stringprep import nodeprep, resourceprep, nameprep
 
+
 try:
 	import winsound # windows-only built-in module for playing wav
 	import win32api
@@ -36,19 +40,12 @@ try:
 except:
 	pass
 
-_ = i18n._
-Q_ = i18n.Q_
-
 special_groups = (_('Transports'), _('Not in Roster'), _('Observers'))
 
 class InvalidFormat(Exception):
 	pass
 
-def parse_jid(jidstring):
-	'''Perform stringprep on all JID fragments from a string
-	and return the full jid'''
-	# This function comes from http://svn.twistedmatrix.com/cvs/trunk/twisted/words/protocols/jabber/jid.py
-
+def decompose_jid(jidstring):
 	user = None
 	server = None
 	resource = None
@@ -80,8 +77,22 @@ def parse_jid(jidstring):
 				# server/resource (with an @ in resource)
 				server = jidstring[0:res_sep]
 				resource = jidstring[res_sep + 1:] or None
+	return user, server, resource
 
-	return prep(user, server, resource)
+def parse_jid(jidstring):
+	'''Perform stringprep on all JID fragments from a string
+	and return the full jid'''
+	# This function comes from http://svn.twistedmatrix.com/cvs/trunk/twisted/words/protocols/jabber/jid.py
+
+	return prep(*decompose_jid(jidstring))
+
+def idn_to_ascii(host):
+	'''convert IDN (Internationalized Domain Names) to ACE (ASCII-compatible encoding)'''
+	labels = idna.dots.split(host)
+	converted_labels = []
+	for label in labels:
+		converted_labels.append(idna.ToASCII(label))
+	return ".".join(converted_labels)
 
 def parse_resource(resource):
 	'''Perform stringprep on resource and return it'''
@@ -179,6 +190,25 @@ def convert_bytes(string):
 		suffix = _('%s B')
 	return suffix % unicode(bytes)
 
+
+def get_contact_dict_for_account(account):
+	''' create a dict of jid, nick -> contact with all contacts of account.
+	Can be used for completion lists'''
+	contacts_dict = {}
+	for jid in gajim.contacts.get_jid_list(account):
+		contact = gajim.contacts.get_contact_with_highest_priority(account,
+				jid)
+		contacts_dict[jid] = contact
+		name = contact.name
+		if contacts_dict.has_key(name):
+			contact1 = contacts_dict[name]
+			del contacts_dict[name]
+			contacts_dict['%s (%s)' % (name, contact1.jid)] = contact1
+			contacts_dict['%s (%s)' % (name, jid)] = contact
+		else:
+			contacts_dict[name] = contact
+	return contacts_dict
+
 def get_uf_show(show, use_mnemonic = False):
 	'''returns a userfriendly string for dnd/xa/chat
 	and makes all strings translatable
@@ -273,6 +303,21 @@ def get_uf_role(role, plural = False):
 		else:
 			role_name = _('Visitor')
 	return role_name
+	
+def get_uf_affiliation(affiliation):
+	'''Get a nice and translated affilition for muc'''
+	if affiliation == 'none': 
+		affiliation_name = Q_('?Group Chat Contact Affiliation:None')
+	elif affiliation == 'owner':
+		affiliation_name = _('Owner')
+	elif affiliation == 'admin':
+		affiliation_name = _('Administrator')
+	elif affiliation == 'member':
+		affiliation_name = _('Member')
+	else: # Argl ! An unknown affiliation !
+		affiliation_name = affiliation.capitalize()
+	return affiliation_name
+
 
 def get_sorted_keys(adict):
 	keys = adict.keys()
@@ -294,8 +339,8 @@ def from_one_line(msg):
 	# to match the regexp that follows it
 
 	# So here match '\\n' but not if you have a '\' before that
-	re = sre.compile(r'(?<!\\)\\n')
-	msg = re.sub('\n', msg)
+	expr = re.compile(r'(?<!\\)\\n')
+	msg = expr.sub('\n', msg)
 	msg = msg.replace('\\\\', '\\')
 	# s12 = 'test\\ntest\\\\ntest'
 	# s13 = re.sub('\n', s12)
@@ -313,7 +358,7 @@ def get_uf_chatstate(chatstate):
 	elif chatstate == 'composing':
 		return _('is composing a message...')
 	elif chatstate == 'paused':
-		#paused means he or she was compoing but has stopped for a while
+		#paused means he or she was composing but has stopped for a while
 		return _('paused composing a message')
 	elif chatstate == 'gone':
 		return _('has closed the chat window or tab')
@@ -344,6 +389,16 @@ def is_in_path(name_of_command, return_abs_path = False):
 	else:
 		return is_in_dir
 
+def exec_command(command):
+	subprocess.Popen(command, shell = True)
+
+def build_command(executable, parameter):
+	# we add to the parameter (can hold path with spaces)
+	# "" so we have good parsing from shell
+	parameter = parameter.replace('"', '\\"') # but first escape "
+	command = '%s "%s"' % (executable, parameter)
+	return command
+
 def launch_browser_mailer(kind, uri):
 	#kind = 'url' or 'mail'
 	if os.name == 'nt':
@@ -360,6 +415,8 @@ def launch_browser_mailer(kind, uri):
 			command = 'gnome-open'
 		elif gajim.config.get('openwith') == 'kfmclient exec':
 			command = 'kfmclient exec'
+		elif gajim.config.get('openwith') == 'exo-open':
+			command = 'exo-open'
 		elif gajim.config.get('openwith') == 'custom':
 			if kind == 'url':
 				command = gajim.config.get('custombrowser')
@@ -367,11 +424,10 @@ def launch_browser_mailer(kind, uri):
 				command = gajim.config.get('custommailapp')
 			if command == '': # if no app is configured
 				return
-		# we add the uri in "" so we have good parsing from shell
-		uri = uri.replace('"', '\\"') # escape "
-		command = command + ' "' + uri + '" &'
-		try: #FIXME: when we require python2.4+ use subprocess module
-			os.system(command)
+
+		command = build_command(command, uri)
+		try:
+			exec_command(command)
 		except:
 			pass
 
@@ -386,15 +442,15 @@ def launch_file_manager(path_to_open):
 			command = 'gnome-open'
 		elif gajim.config.get('openwith') == 'kfmclient exec':
 			command = 'kfmclient exec'
+		elif gajim.config.get('openwith') == 'exo-open':
+			command = 'exo-open'
 		elif gajim.config.get('openwith') == 'custom':
 			command = gajim.config.get('custom_file_manager')
 		if command == '': # if no app is configured
 			return
-		# we add the path in "" so we have good parsing from shell
-		path_to_open = path_to_open.replace('"', '\\"') # escape "
-		command = command + ' "' + path_to_open + '" &'
-		try: #FIXME: when we require python2.4+ use subprocess module
-			os.system(command)
+		command = build_command(command, path_to_open)
+		try:
+			exec_command(command)
 		except:
 			pass
 
@@ -402,6 +458,9 @@ def play_sound(event):
 	if not gajim.config.get('sounds_on'):
 		return
 	path_to_soundfile = gajim.config.get_per('soundevents', event, 'path')
+	play_sound_file(path_to_soundfile)
+
+def play_sound_file(path_to_soundfile):
 	if path_to_soundfile == 'beep':
 		print '\a' # make a speaker beep
 		return
@@ -417,11 +476,8 @@ def play_sound(event):
 		if gajim.config.get('soundplayer') == '':
 			return
 		player = gajim.config.get('soundplayer')
-		# we add the path in "" so we have good parsing from shell
-		path_to_soundfile = path_to_soundfile.replace('"', '\\"') # escape "
-		command = player + ' "' + path_to_soundfile + '" &'
-		#FIXME: when we require 2.4+ use subprocess module
-		os.system(command)
+		command = build_command(player, path_to_soundfile)
+		exec_command(command)
 
 def get_file_path_from_dnd_dropped_uri(uri):
 	path = urllib.url2pathname(uri) # escape special chars
@@ -445,14 +501,6 @@ def from_xs_boolean_to_python_boolean(value):
 
 	return val
 
-def ensure_unicode_string(s):
-	# py23 u'abc'.decode('utf-8') raises
-	# python24 does not. if python23 is ooold we can remove this func
-	# FIXME: remove this when we abandon py23
-	if isinstance(s, str):
-		s = s.decode('utf-8')
-	return s
-
 def get_xmpp_show(show):
 	if show in ('online', 'offline'):
 		return None
@@ -495,9 +543,9 @@ def get_global_status():
 
 def get_icon_name_to_show(contact, account = None):
 	'''Get the icon name to show in online, away, requested, ...'''
-	if account and gajim.awaiting_events[account].has_key(contact.jid):
+	if account and gajim.events.get_nb_roster_events(account, contact.jid):
 		return 'message'
-	if account and gajim.awaiting_events[account].has_key(
+	if account and gajim.events.get_nb_roster_events(account,
 	contact.get_full_jid()):
 		return 'message'
 	if contact.jid.find('@') <= 0: # if not '@' or '@' starts the jid ==> agent
@@ -513,8 +561,10 @@ def get_icon_name_to_show(contact, account = None):
 
 def decode_string(string):
 	'''try to decode (to make it Unicode instance) given string'''
+	if isinstance(string, unicode):
+		return string
 	# by the time we go to iso15 it better be the one else we show bad characters
-	encodings = (sys.getfilesystemencoding(), 'utf-8', 'iso-8859-15')
+	encodings = (locale.getpreferredencoding(), 'utf-8', 'iso-8859-15')
 	for encoding in encodings:
 		try:
 			string = string.decode(encoding)
@@ -524,6 +574,14 @@ def decode_string(string):
 
 	return string
 
+def ensure_utf8_string(string):
+	'''make sure string is in UTF-8'''
+	try:
+		string = decode_string(string).encode('utf-8')
+	except:
+		pass
+	return string
+
 def get_windows_reg_env(varname, default=''):
 	'''asks for paths commonly used but not exposed as ENVs
 	in english Windows 2003 those are:
@@ -562,7 +620,7 @@ r'Software\Microsoft\Windows\CurrentVersion\Explorer\User Shell Folders')
 	finally:
 		win32api.RegCloseKey(rkey)
 	return val
- 
+
 def get_my_pictures_path():
 	'''windows-only atm. [Unix lives in the past]'''
 	return get_windows_reg_env('My Pictures')
@@ -581,7 +639,6 @@ def get_documents_path():
 		path = os.path.expanduser('~')
 	return path
 
-# moved from connection.py
 def get_full_jid_from_iq(iq_obj):
 	'''return the full jid (with resource) from an iq as unicode'''
 	return parse_jid(str(iq_obj.getFrom()))
@@ -640,7 +697,8 @@ def get_os_info():
 			(2, 4, 0): 'NT',
 			(2, 5, 0): '2000',
 			(2, 5, 1): 'XP',
-			(2, 5, 2): '2003'
+			(2, 5, 2): '2003',
+			(2, 6, 0): 'Vista',
 		}
 		if win_version.has_key(ver_format):
 			return 'Windows' + ' ' + win_version[ver_format]
@@ -656,6 +714,7 @@ def get_os_info():
 			output = temp_failure_retry(child_stdout.readline).strip()
 			child_stdout.close()
 			child_stdin.close()
+			os.wait()
 			# some distros put n/a in places, so remove those
 			output = output.replace('n/a', '').replace('N/A', '')
 			return output
@@ -673,8 +732,11 @@ def get_os_info():
 					text = fd.readline().strip() # get only first line
 					fd.close()
 					if path_to_file.endswith('version'):
-						# sourcemage_version has all the info we need
-						if not os.path.basename(path_to_file).startswith('sourcemage'):
+						# sourcemage_version and slackware-version files
+						# have all the info we need (name and version of distro)
+						if not os.path.basename(path_to_file).startswith(
+						'sourcemage') or not\
+						os.path.basename(path_to_file).startswith('slackware'):
 							text = distro_name + ' ' + text
 					elif path_to_file.endswith('aurox-release'):
 						# file doesn't have version
@@ -698,27 +760,185 @@ def sanitize_filename(filename):
 		filename = filename.replace('?', '_').replace(':', '_')\
 			.replace('\\', '_').replace('"', "'").replace('|', '_')\
 			.replace('*', '_').replace('<', '_').replace('>', '_')
-	   # 48 is the limit
+		# 48 is the limit
 		if len(filename) > 48:
 			extension = filename.split('.')[-1]
 			filename = filename[0:48]
 	
 	return filename
 
-def allow_showing_notification(account):
+def allow_showing_notification(account, type = 'notify_on_new_message',
+advanced_notif_num = None, is_first_message = True):
 	'''is it allowed to show nofication?
-	check OUR status and if we allow notifications for that status'''
+	check OUR status and if we allow notifications for that status
+	type is the option that need to be True e.g.: notify_on_signing
+	is_first_message: set it to false when it's not the first message'''
+	if advanced_notif_num is not None:
+		popup = gajim.config.get_per('notifications', str(advanced_notif_num),
+			'popup')
+		if popup == 'yes':
+			return True
+		if popup == 'no':
+			return False
+	if type and (not gajim.config.get(type) or not is_first_message):
+		return False
 	if gajim.config.get('autopopupaway'): # always show notification
 		return True
 	if gajim.connections[account].connected in (2, 3): # we're online or chat
 		return True
 	return False
 
-def allow_popup_window(account):
+def allow_popup_window(account, advanced_notif_num = None):
 	'''is it allowed to popup windows?'''
+	if advanced_notif_num != None:
+		popup = gajim.config.get_per('notifications', str(advanced_notif_num),
+			'auto_open')
+		if popup == 'yes':
+			return True
+		if popup == 'no':
+			return False
 	autopopup = gajim.config.get('autopopup')
 	autopopupaway = gajim.config.get('autopopupaway')
 	if autopopup and (autopopupaway or \
 	gajim.connections[account].connected in (2, 3)): # we're online or chat
 		return True
 	return False
+
+def allow_sound_notification(sound_event, advanced_notif_num = None):
+	if advanced_notif_num is not None:
+		sound = gajim.config.get_per('notifications', str(advanced_notif_num),
+			'sound')
+		if sound == 'yes':
+			return True
+		if sound == 'no':
+			return False
+	if gajim.config.get_per('soundevents', sound_event, 'enabled'):
+		return True
+	return False
+
+def get_chat_control(account, contact):
+	full_jid_with_resource = contact.jid
+	if contact.resource:
+		full_jid_with_resource += '/' + contact.resource
+	highest_contact = gajim.contacts.get_contact_with_highest_priority(
+		account, contact.jid)
+	# Look for a chat control that has the given resource, or default to
+	# one without resource
+	ctrl = gajim.interface.msg_win_mgr.get_control(full_jid_with_resource,
+		account)
+	if ctrl:
+		return ctrl
+	elif not highest_contact or not highest_contact.resource:
+		# unknow contact or offline message
+		return gajim.interface.msg_win_mgr.get_control(contact.jid, account)
+	elif highest_contact and contact.resource != \
+	highest_contact.resource:
+		return None
+	return gajim.interface.msg_win_mgr.get_control(contact.jid, account)
+
+def reduce_chars_newlines(text, max_chars = 0, max_lines = 0):
+	'''Cut the chars after 'max_chars' on each line
+	and show only the first 'max_lines'.
+	If any of the params is not present (None or 0) the action
+	on it is not performed'''
+
+	def _cut_if_long(string):
+		if len(string) > max_chars:
+			string = string[:max_chars - 3] + '...'
+		return string
+
+	if isinstance(text, str):
+		text = text.decode('utf-8')
+
+	if max_lines == 0:
+		lines = text.split('\n')
+	else:
+		lines = text.split('\n', max_lines)[:max_lines]
+	if max_chars > 0:
+		if lines:
+			lines = map(lambda e: _cut_if_long(e), lines)
+	if lines:
+		reduced_text = reduce(lambda e, e1: e + '\n' + e1, lines)
+	else:
+		reduced_text = ''
+	return reduced_text
+
+def get_notification_icon_tooltip_text():
+	text = None
+	unread_chat = gajim.events.get_nb_events(types = ['printed_chat',
+		'chat'])
+	unread_single_chat = gajim.events.get_nb_events(types = ['normal'])
+	unread_gc = gajim.events.get_nb_events(types = ['printed_gc_msg',
+		'gc_msg'])
+	unread_pm = gajim.events.get_nb_events(types = ['printed_pm', 'pm'])
+
+	accounts = get_accounts_info()
+
+	if unread_chat or unread_single_chat or unread_gc or unread_pm:
+		text = 'Gajim '
+		awaiting_events = unread_chat + unread_single_chat + unread_gc + unread_pm
+		if awaiting_events == unread_chat or awaiting_events == unread_single_chat \
+			or awaiting_events == unread_gc or awaiting_events == unread_pm:
+			# This condition is like previous if but with xor... 
+			# Print in one line
+			text += '-'
+		else:
+			# Print in multiple lines
+			text += '\n   '
+		if unread_chat:
+			text += ngettext(
+				' %d unread message',
+				' %d unread messages',
+				unread_chat, unread_chat, unread_chat)
+			text += '\n   '
+		if unread_single_chat:
+			text += ngettext(
+				' %d unread single message',
+				' %d unread single messages',
+				unread_single_chat, unread_single_chat, unread_single_chat)
+			text += '\n   '
+		if unread_gc:
+			text += ngettext(
+				' %d unread group chat message',
+				' %d unread group chat messages',
+				unread_gc, unread_gc, unread_gc)
+			text += '\n   '
+		if unread_pm:
+			text += ngettext(
+				' %d unread private message',
+				' %d unread private messages',
+				unread_pm, unread_pm, unread_pm)
+			text += '\n   '
+		text = text[:-4] # remove latest '\n   '
+	elif len(accounts) > 1:
+		text = _('Gajim')
+	elif len(accounts) == 1:
+		message = accounts[0]['status_line']
+		message = reduce_chars_newlines(message, 100, 1)
+		text = _('Gajim - %s') % message
+	else:
+		text = _('Gajim - %s') % get_uf_show('offline')
+		
+	return text
+
+def get_accounts_info():
+	'''helper for notification icon tooltip'''
+	accounts = []
+	accounts_list = gajim.contacts.get_accounts()
+	accounts_list.sort()
+	for account in accounts_list:
+		status_idx = gajim.connections[account].connected
+		# uncomment the following to hide offline accounts
+		# if status_idx == 0: continue
+		status = gajim.SHOW_LIST[status_idx]
+		message = gajim.connections[account].status
+		single_line = get_uf_show(status)
+		if message is None:
+			message = ''
+		else:
+			message = message.strip()
+		if message != '':
+			single_line += ': ' + message
+		accounts.append({'name': account, 'status_line': single_line, 
+				'show': status, 'message': message})
+	return accounts
