@@ -107,8 +107,11 @@ class ProxyResolver:
 		query = iq.setTag('query')
 		query.setNamespace(common.xmpp.NS_BYTESTREAM)
 		query.setAttr('sid',  self.sid)
-		activate = query.setTag('activate')
+		
+		query.setTag('activate')
+		# activate = query.setTag('activate')
 		# activate.setData(self.jid + "/" + self.sid)
+
 		if self.active_connection:
 			self.active_connection.send(iq)
 			self.state = S_ACTIVATED
@@ -186,6 +189,9 @@ class HostTester(Socks5, IdleObject):
 	
 	def connect(self):
 		''' create the socket and plug it to the idlequeue '''
+		if self.host is None:
+			self.on_failure()
+			return None
 		self._sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
 		self._sock.setblocking(False)
 		self.fd = self._sock.fileno()
