@@ -1,17 +1,8 @@
 ##	conversation_textview.py
 ##
-## Contributors for this file:
-##	- Yann Le Boulanger <asterix@lagaule.org>
-##	- Nikos Kouremenos <kourem@gmail.com>
-##
-## Copyright (C) 2003-2004 Yann Le Boulanger <asterix@lagaule.org>
-##                         Vincent Hanquez <tab@snarc.org>
-## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
-##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
-##                    Dimitur Kirov <dkirov@gmail.com>
-##                    Travis Shirk <travis@pobox.com>
-##                    Norman Rasmussen <norman@rasmussen.co.za>
+## Copyright (C) 2005-2006 Yann Le Boulanger <asterix@lagaule.org>
+## Copyright (C) 2005-2006 Nikos Kouremenos <kourem@gmail.com>
+## Copyright (C) 2005-2006 Travis Shirk <travis@pobox.com>
 ##
 ## This program is free software; you can redistribute it and/or modify
 ## it under the terms of the GNU General Public License as published
@@ -24,11 +15,10 @@
 ##
 
 import gtk
-import gtk.glade
 import pango
 import gobject
 import time
-import sys
+import os
 import tooltips
 import dialogs
 import locale
@@ -36,37 +26,46 @@ import locale
 import gtkgui_helpers
 from common import gajim
 from common import helpers
-from common import i18n
 from calendar import timegm
+from common.fuzzyclock import FuzzyClock
 
-_ = i18n._
-APP = i18n.APP
-gtk.glade.bindtextdomain(APP, i18n.DIR)
-gtk.glade.textdomain(APP)
+from htmltextview import HtmlTextView
+from common.exceptions import GajimGeneralException
 
 class ConversationTextview:
 	'''Class for the conversation textview (where user reads already said messages)
 	for chat/groupchat windows'''
-	def __init__(self, account):
-		# no need to inherit TextView, use it as property is safer
-		self.tv = gtk.TextView()
+	
+	path_to_file = os.path.join(gajim.DATA_DIR, 'pixmaps', 'muc_separator.png')
+	FOCUS_OUT_LINE_PIXBUF = gtk.gdk.pixbuf_new_from_file(path_to_file)
+
+	def __init__(self, account, used_in_history_window = False):
+		'''if used_in_history_window is True, then we do not show
+		Clear menuitem in context menu'''
+		self.used_in_history_window = used_in_history_window
+		
+		# no need to inherit TextView, use it as atrribute is safer
+		self.tv = HtmlTextView()
+		self.tv.html_hyperlink_handler = self.html_hyperlink_handler
 
 		# set properties
 		self.tv.set_border_width(1)
 		self.tv.set_accepts_tab(True)
 		self.tv.set_editable(False)
 		self.tv.set_cursor_visible(False)
-		self.tv.set_wrap_mode(gtk.WRAP_WORD)
+		self.tv.set_wrap_mode(gtk.WRAP_WORD_CHAR)
 		self.tv.set_left_margin(2)
 		self.tv.set_right_margin(2)
 		self.handlers = {}
 
 		# connect signals
-		id = self.tv.connect('motion_notify_event', self.on_textview_motion_notify_event)
+		id = self.tv.connect('motion_notify_event',
+			self.on_textview_motion_notify_event)
 		self.handlers[id] = self.tv
 		id = self.tv.connect('populate_popup', self.on_textview_populate_popup)
 		self.handlers[id] = self.tv
-		id = self.tv.connect('button_press_event', self.on_textview_button_press_event)
+		id = self.tv.connect('button_press_event',
+			self.on_textview_button_press_event)
 		self.handlers[id] = self.tv
 
 		self.account = account
@@ -89,13 +88,21 @@ class ConversationTextview:
 		color = gajim.config.get('statusmsgcolor')
 		self.tagStatus.set_property('foreground', color)
 
+		colors = gajim.config.get('gc_nicknames_colors')
+		colors = colors.split(':')
+		for color in xrange(len(colors)):
+			tagname = 'gc_nickname_color_' + str(color)
+			tag = buffer.create_tag(tagname)
+			color = colors[color]
+			tag.set_property('foreground', color)
+
 		tag = buffer.create_tag('marked')
 		color = gajim.config.get('markedmsgcolor')
 		tag.set_property('foreground', color)
 		tag.set_property('weight', pango.WEIGHT_BOLD)
 
 		tag = buffer.create_tag('time_sometimes')
-		tag.set_property('foreground', 'grey')
+		tag.set_property('foreground', 'darkgrey')
 		tag.set_property('scale', pango.SCALE_SMALL)
 		tag.set_property('justification', gtk.JUSTIFY_CENTER)
 
@@ -106,18 +113,18 @@ class ConversationTextview:
 		color = gajim.config.get('restored_messages_color')
 		tag.set_property('foreground', color)
 
-		tag = buffer.create_tag('url')
+		self.tagURL = buffer.create_tag('url')
 		color = gajim.config.get('urlmsgcolor')
-		tag.set_property('foreground', color)
-		tag.set_property('underline', pango.UNDERLINE_SINGLE)
-		id = tag.connect('event', self.hyperlink_handler, 'url')
-		self.handlers[id] = tag
+		self.tagURL.set_property('foreground', color)
+		self.tagURL.set_property('underline', pango.UNDERLINE_SINGLE)
+		id = self.tagURL.connect('event', self.hyperlink_handler, 'url')
+		self.handlers[id] = self.tagURL
 
-		tag = buffer.create_tag('mail')
-		tag.set_property('foreground', color)
-		tag.set_property('underline', pango.UNDERLINE_SINGLE)
-		id = tag.connect('event', self.hyperlink_handler, 'mail')
-		self.handlers[id] = tag
+		self.tagMail = buffer.create_tag('mail')
+		self.tagMail.set_property('foreground', color)
+		self.tagMail.set_property('underline', pango.UNDERLINE_SINGLE)
+		id = self.tagMail.connect('event', self.hyperlink_handler, 'mail')
+		self.handlers[id] = self.tagMail
 
 		tag = buffer.create_tag('bold')
 		tag.set_property('weight', pango.WEIGHT_BOLD)
@@ -130,7 +137,13 @@ class ConversationTextview:
 
 		buffer.create_tag('focus-out-line', justification = gtk.JUSTIFY_CENTER)
 
+		self.allow_focus_out_line = True
+		# holds the iter's offset which points to the end of --- line
+		self.focus_out_end_iter_offset = None
+
 		self.line_tooltip = tooltips.BaseTooltip()
+		# use it for hr too
+		self.tv.focus_out_line_pixbuf = ConversationTextview.FOCUS_OUT_LINE_PIXBUF
 
 	def del_handlers(self):
 		for i in self.handlers.keys():
@@ -138,7 +151,7 @@ class ConversationTextview:
 				self.handlers[i].disconnect(i)
 		del self.handlers
 		self.tv.destroy()
-		#TODO
+		#FIXME:
 		# self.line_tooltip.destroy()
 	
 	def update_tags(self):
@@ -146,6 +159,8 @@ class ConversationTextview:
 		self.tagOut.set_property('foreground', gajim.config.get('outmsgcolor'))
 		self.tagStatus.set_property('foreground',
 			gajim.config.get('statusmsgcolor'))
+		self.tagURL.set_property('foreground', gajim.config.get('urlmsgcolor'))
+		self.tagMail.set_property('foreground', gajim.config.get('urlmsgcolor'))
 
 	def at_the_end(self):
 		buffer = self.tv.get_buffer()
@@ -180,9 +195,69 @@ class ConversationTextview:
 	def scroll_to_end_iter(self):
 		buffer = self.tv.get_buffer()
 		end_iter = buffer.get_end_iter()
+		if not end_iter:
+			return False
 		self.tv.scroll_to_iter(end_iter, 0, False, 1, 1)
 		return False # when called in an idle_add, just do it once
 
+	def show_focus_out_line(self):
+		if not self.allow_focus_out_line:
+			# if room did not receive focus-in from the last time we added
+			# --- line then do not readd
+			return
+
+		print_focus_out_line = False
+		buffer = self.tv.get_buffer()
+
+		if self.focus_out_end_iter_offset is None:
+			# this happens only first time we focus out on this room
+			print_focus_out_line = True
+
+		else:
+			if self.focus_out_end_iter_offset != buffer.get_end_iter().\
+			get_offset():
+				# this means after last-focus something was printed
+				# (else end_iter's offset is the same as before)
+				# only then print ---- line (eg. we avoid printing many following
+				# ---- lines)
+				print_focus_out_line = True
+
+		if print_focus_out_line and buffer.get_char_count() > 0:
+			buffer.begin_user_action()
+
+			# remove previous focus out line if such focus out line exists
+			if self.focus_out_end_iter_offset is not None:
+				end_iter_for_previous_line = buffer.get_iter_at_offset(
+					self.focus_out_end_iter_offset)
+				begin_iter_for_previous_line = end_iter_for_previous_line.copy()
+				# img_char+1 (the '\n')
+				begin_iter_for_previous_line.backward_chars(2)
+
+				# remove focus out line
+				buffer.delete(begin_iter_for_previous_line,
+					end_iter_for_previous_line)
+
+			# add the new focus out line
+			end_iter = buffer.get_end_iter()
+			buffer.insert(end_iter, '\n')
+			buffer.insert_pixbuf(end_iter, 
+				ConversationTextview.FOCUS_OUT_LINE_PIXBUF)
+
+			end_iter = buffer.get_end_iter()
+			before_img_iter = end_iter.copy()
+			before_img_iter.backward_char() # one char back (an image also takes one char)
+			buffer.apply_tag_by_name('focus-out-line', before_img_iter, end_iter)
+
+			self.allow_focus_out_line = False
+
+			# update the iter we hold to make comparison the next time
+			self.focus_out_end_iter_offset = buffer.get_end_iter().get_offset()
+
+			buffer.end_user_action()
+
+			# scroll to the end (via idle in case the scrollbar has appeared)
+			gobject.idle_add(self.scroll_to_end)
+
 	def show_line_tooltip(self):
 		pointer = self.tv.get_pointer()
 		x, y = self.tv.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT, pointer[0],
@@ -197,7 +272,6 @@ class ConversationTextview:
 		if over_line and not self.line_tooltip.win:
 			# check if the current pointer is still over the line
 			position = self.tv.window.get_origin()
-			win = self.tv.get_toplevel()
 			self.line_tooltip.show_tooltip(_('Text below this line is what has '
 			'been said since the last time you paid attention to this group chat'),	8, position[1] + pointer[1])
 
@@ -237,6 +311,7 @@ class ConversationTextview:
 		buffer = self.tv.get_buffer()
 		start, end = buffer.get_bounds()
 		buffer.delete(start, end)
+		self.focus_out_end_iter_offset = None
 
 	def visit_url_from_menuitem(self, widget, link):
 		'''basically it filters out the widget instance'''
@@ -244,19 +319,29 @@ class ConversationTextview:
 
 	def on_textview_populate_popup(self, textview, menu):
 		'''we override the default context menu and we prepend Clear
+		(only if used_in_history_window is False)
 		and if we have sth selected we show a submenu with actions on the phrase
 		(see on_conversation_textview_button_press_event)'''
-		item = gtk.SeparatorMenuItem()
-		menu.prepend(item)
-		item = gtk.ImageMenuItem(gtk.STOCK_CLEAR)
-		menu.prepend(item)
-		id = item.connect('activate', self.clear)
-		self.handlers[id] = item
+
+		separator_menuitem_was_added = False
+		if not self.used_in_history_window:
+			item = gtk.SeparatorMenuItem()
+			menu.prepend(item)
+			separator_menuitem_was_added = True
+
+			item = gtk.ImageMenuItem(gtk.STOCK_CLEAR)
+			menu.prepend(item)
+			id = item.connect('activate', self.clear)
+			self.handlers[id] = item
+
 		if self.selected_phrase:
-			s = self.selected_phrase
-			if len(s) > 25:
-				s = s[:21] + '...'
-			item = gtk.MenuItem(_('Actions for "%s"') % s)
+			if not separator_menuitem_was_added:
+				item = gtk.SeparatorMenuItem()
+				menu.prepend(item)
+
+			self.selected_phrase = helpers.reduce_chars_newlines(
+				self.selected_phrase, 25, 2)
+			item = gtk.MenuItem(_('_Actions for "%s"') % self.selected_phrase)
 			menu.prepend(item)
 			submenu = gtk.Menu()
 			item.set_submenu(submenu)
@@ -288,19 +373,20 @@ class ConversationTextview:
 				self.handlers[id] = item
 			else:
 				if dict_link.find('%s') == -1:
-					#we must have %s in the url if not WIKTIONARY
+					# we must have %s in the url if not WIKTIONARY
 					item = gtk.MenuItem(_('Dictionary URL is missing an "%s" and it is not WIKTIONARY'))
 					item.set_property('sensitive', False)
 				else:
 					link = dict_link % self.selected_phrase
-					id = item.connect('activate', self.visit_url_from_menuitem, link)
+					id = item.connect('activate', self.visit_url_from_menuitem,
+						link)
 					self.handlers[id] = item
 			submenu.append(item)
 
 
 			search_link = gajim.config.get('search_engine')
 			if search_link.find('%s') == -1:
-				#we must have %s in the url
+				# we must have %s in the url
 				item = gtk.MenuItem(_('Web Search URL is missing an "%s"'))
 				item.set_property('sensitive', False)
 			else:
@@ -309,19 +395,22 @@ class ConversationTextview:
 				id = item.connect('activate', self.visit_url_from_menuitem, link)
 				self.handlers[id] = item
 			submenu.append(item)
+			
+			item = gtk.MenuItem(_('Open as _Link'))
+			id = item.connect('activate', self.visit_url_from_menuitem, link)
+			self.handlers[id] = item
+			submenu.append(item)
 
 		menu.show_all()
 
 	def on_textview_button_press_event(self, widget, event):
 		# If we clicked on a taged text do NOT open the standard popup menu
 		# if normal text check if we have sth selected
-
-		self.selected_phrase = ''
+		self.selected_phrase = '' # do not move belove event button check!
 
 		if event.button != 3: # if not right click
 			return False
 
-		win = self.tv.get_window(gtk.TEXT_WINDOW_TEXT)
 		x, y = self.tv.window_to_buffer_coords(gtk.TEXT_WINDOW_TEXT,
 			int(event.x), int(event.y))
 		iter = self.tv.get_iter_at_location(x, y)
@@ -331,7 +420,7 @@ class ConversationTextview:
 		if tags: # we clicked on sth special (it can be status message too)
 			for tag in tags:
 				tag_name = tag.get_property('name')
-				if 'url' in tag_name or 'mail' in tag_name:
+				if tag_name in ('url', 'mail'):
 					return True # we block normal context menu
 
 		# we check if sth was selected and if it was we assign
@@ -354,18 +443,16 @@ class ConversationTextview:
 	def on_start_chat_activate(self, widget, jid):
 		gajim.interface.roster.new_chat_from_jid(self.account, jid)
 
-	def on_join_group_chat_menuitem_activate(self, widget, jid):
-		room, server = jid.split('@')
-		if gajim.interface.instances[self.account].has_key('join_gc'):
+	def on_join_group_chat_menuitem_activate(self, widget, room_jid):
+		if 'join_gc' in gajim.interface.instances[self.account]:
 			instance = gajim.interface.instances[self.account]['join_gc']
-			instance.xml.get_widget('server_entry').set_text(server)
-			instance.xml.get_widget('room_entry').set_text(room)
+			instance.xml.get_widget('room_jid_entry').set_text(room_jid)
 			gajim.interface.instances[self.account]['join_gc'].window.present()
 		else:
 			try:
 				gajim.interface.instances[self.account]['join_gc'] = \
-				dialogs.JoinGroupchatWindow(self.account, server, room)
-			except RuntimeError:
+				dialogs.JoinGroupchatWindow(self.account, room_jid)
+			except GajimGeneralException:
 				pass
 
 	def on_add_to_roster_activate(self, widget, jid):
@@ -387,6 +474,13 @@ class ConversationTextview:
 			childs[6].hide() # join group chat
 			childs[7].hide() # add to roster
 		else: # It's a mail or a JID
+			# load muc icon
+			join_group_chat_menuitem = xml.get_widget('join_group_chat_menuitem')
+			muc_icon = gajim.interface.roster.load_icon('muc_active')
+			if muc_icon: 
+				join_group_chat_menuitem.set_image(muc_icon) 
+
+			text = text.lower()
 			id = childs[2].connect('activate', self.on_copy_link_activate, text)
 			self.handlers[id] = childs[2]
 			id = childs[3].connect('activate', self.on_open_link_activate, kind, text)
@@ -434,6 +528,15 @@ class ConversationTextview:
 				# we launch the correct application
 				helpers.launch_browser_mailer(kind, word)
 
+	def html_hyperlink_handler(self, texttag, widget, event, iter, kind, href):
+		if event.type == gtk.gdk.BUTTON_PRESS:
+			if event.button == 3: # right click
+				self.make_link_menu(event, kind, href)
+			else:
+				# we launch the correct application
+				helpers.launch_browser_mailer(kind, href)
+
+
 	def detect_and_print_special_text(self, otext, other_tags):
 		'''detects special text (emots & links & formatting)
 		prints normal text before any special text it founts,
@@ -481,15 +584,20 @@ class ConversationTextview:
 		possible_emot_ascii_caps = special_text.upper() # emoticons keys are CAPS
 		if gajim.config.get('emoticons_theme') and \
 		possible_emot_ascii_caps in gajim.interface.emoticons.keys():
-			#it's an emoticon
+			# it's an emoticon
 			emot_ascii = possible_emot_ascii_caps
 			end_iter = buffer.get_end_iter()
 			anchor = buffer.create_child_anchor(end_iter)
 			img = gtk.Image()
-			img.set_from_file(gajim.interface.emoticons[emot_ascii])
+			animations = gajim.interface.emoticons_animations
+			if not emot_ascii in animations:
+				animations[emot_ascii] = gtk.gdk.PixbufAnimation(
+					gajim.interface.emoticons[emot_ascii])
+			img.set_from_animation(animations[emot_ascii])
 			img.show()
-			#add with possible animation
+			# add with possible animation
 			self.tv.add_child_at_anchor(img, anchor)
+		#FIXME: one day, somehow sync with regexp in gajim.py
 		elif special_text.startswith('http://') or \
 			special_text.startswith('www.') or \
 			special_text.startswith('ftp://') or \
@@ -501,15 +609,12 @@ class ConversationTextview:
 			special_text.startswith('irc://') or \
 			special_text.startswith('sip:') or \
 			special_text.startswith('magnet:'):
-			#it's a url
+			# it's a url
 			tags.append('url')
 			use_other_tags = False
-		elif special_text.startswith('mailto:'):
-			#it's a mail
-			tags.append('mail')
-			use_other_tags = False
-		elif gajim.interface.sth_at_sth_dot_sth_re.match(special_text):
-			#it's a mail
+		elif special_text.startswith('mailto:') or \
+		gajim.interface.sth_at_sth_dot_sth_re.match(special_text):
+			# it's a mail
 			tags.append('mail')
 			use_other_tags = False
 		elif special_text.startswith('*'): # it's a bold text
@@ -566,17 +671,12 @@ class ConversationTextview:
 	def print_empty_line(self):
 		buffer = self.tv.get_buffer()
 		end_iter = buffer.get_end_iter()
-		buffer.insert(end_iter, '\n')
+		buffer.insert_with_tags_by_name(end_iter, '\n', 'eol')
 
 	def print_conversation_line(self, text, jid, kind, name, tim,
-			other_tags_for_name = [], other_tags_for_time = [],
-			other_tags_for_text = [], subject = None):
+	other_tags_for_name = [], other_tags_for_time = [], other_tags_for_text = [],
+	subject = None, old_kind = None, xhtml = None):
 		'''prints 'chat' type messages'''
-		if kind == 'status' and not gajim.config.get('print_status_in_chats'):
-				return
-		# kind = info, we print things as if it was a status: same color, ...
-		if kind == 'info':
-			kind = 'status'
 		buffer = self.tv.get_buffer()
 		buffer.begin_user_action()
 		end_iter = buffer.get_end_iter()
@@ -585,56 +685,57 @@ class ConversationTextview:
 			at_the_end = True
 
 		if buffer.get_char_count() > 0:
-			buffer.insert(end_iter, '\n')
+			buffer.insert_with_tags_by_name(end_iter, '\n', 'eol')
 		if kind == 'incoming_queue':
 			kind = 'incoming'
+		if old_kind == 'incoming_queue':
+			old_kind = 'incoming'
 		# print the time stamp
 		if not tim:
 			# We don't have tim for outgoing messages...
 			tim = time.localtime()
-		if gajim.config.get('print_time') == 'always':
-			before_str = gajim.config.get('before_time')
-			after_str = gajim.config.get('after_time')
-			# get difference in days since epoch (86400 = 24*3600)
-			# number of days since epoch for current time (in GMT) -
-			# number of days since epoch for message (in GMT)
-			diff_day = int(timegm(time.localtime())) / 86400 - int(timegm(tim)) / 86400
-			if diff_day == 0:
-				day_str = ''
-			elif diff_day == 1:
-				day_str = _('Yesterday')
-			else:
-				#the number is >= 2
-				# %i is day in year (1-365), %d (1-31) we want %i
-				day_str = _('%i days ago') % diff_day
-			format = before_str
-			if day_str:
-				format += day_str + ' '
-			format += '%X' + after_str
-			# format comes as unicode, because of day_str.
-			# we convert it to the encoding that we want
-			tim_format = time.strftime(format, tim).encode('utf-8')
-			buffer.insert_with_tags_by_name(end_iter, tim_format + ' ',
+		current_print_time = gajim.config.get('print_time')
+		if current_print_time == 'always' and kind != 'info':
+			timestamp_str = self.get_time_to_show(tim)
+			timestamp = time.strftime(timestamp_str, tim)
+			buffer.insert_with_tags_by_name(end_iter, timestamp,
 				*other_tags_for_time)
-		elif gajim.config.get('print_time') == 'sometimes':
+		elif current_print_time == 'sometimes' and kind != 'info':
 			every_foo_seconds = 60 * gajim.config.get(
 				'print_ichat_every_foo_minutes')
 			seconds_passed = time.mktime(tim) - self.last_time_printout
 			if seconds_passed > every_foo_seconds:
 				self.last_time_printout = time.mktime(tim)
 				end_iter = buffer.get_end_iter()
-				tim_format = time.strftime('%H:%M', tim).decode(
-					locale.getpreferredencoding())
+				if gajim.config.get('print_time_fuzzy') > 0:
+					fc = FuzzyClock()
+					fc.setTime(time.strftime('%H:%M', tim))
+					ft = fc.getFuzzyTime(gajim.config.get('print_time_fuzzy'))
+					tim_format = ft.decode(locale.getpreferredencoding())
+				else:
+					tim_format = self.get_time_to_show(tim)
 				buffer.insert_with_tags_by_name(end_iter, tim_format + '\n',
 					'time_sometimes')
+		# kind = info, we print things as if it was a status: same color, ...
+		if kind == 'info':
+			kind = 'status'
 		other_text_tag = self.detect_other_text_tag(text, kind)
 		text_tags = other_tags_for_text[:] # create a new list
 		if other_text_tag:
+			# note that color of /me may be overwritten in gc_control
 			text_tags.append(other_text_tag)
 		else: # not status nor /me
-			self.print_name(name, kind, other_tags_for_name)
+			if gajim.config.get(
+				'chat_merge_consecutive_nickname'):
+				if kind != old_kind:
+					self.print_name(name, kind, other_tags_for_name)
+				else:
+					self.print_real_text(gajim.config.get(
+						'chat_merge_consecutive_nickname_indent'))
+			else:
+				self.print_name(name, kind, other_tags_for_name)
 		self.print_subject(subject)
-		self.print_real_text(text, text_tags, name)
+		self.print_real_text(text, text_tags, name, xhtml)
 
 		# scroll to the end of the textview
 		if at_the_end or kind == 'outgoing':
@@ -644,6 +745,35 @@ class ConversationTextview:
 
 		buffer.end_user_action()
 
+	def get_time_to_show(self, tim):
+		'''Get the time, with the day before if needed and return it.
+		It DOESN'T format a fuzzy time'''
+		format = ''
+		# get difference in days since epoch (86400 = 24*3600)
+		# number of days since epoch for current time (in GMT) -
+		# number of days since epoch for message (in GMT)
+		diff_day = int(timegm(time.localtime())) / 86400 -\
+			int(timegm(tim)) / 86400
+		if diff_day == 0:
+			day_str = ''
+		elif diff_day == 1:
+			day_str = _('Yesterday')
+		else:
+			#the number is >= 2
+			# %i is day in year (1-365), %d (1-31) we want %i
+			day_str = _('%i days ago') % diff_day
+		if day_str:
+			format += day_str + ' '
+		timestamp_str = gajim.config.get('time_stamp')
+		timestamp_str = helpers.from_one_line(timestamp_str)
+		format += timestamp_str
+		tim_format = time.strftime(format, tim)
+		if locale.getpreferredencoding() != 'KOI8-R':
+			# if tim_format comes as unicode because of day_str.
+			# we convert it to the encoding that we want (and that is utf-8)
+			tim_format = helpers.ensure_utf8_string(tim_format)
+		return tim_format
+
 	def detect_other_text_tag(self, text, kind):
 		if kind == 'status':
 			return kind
@@ -657,7 +787,9 @@ class ConversationTextview:
 			name_tags = other_tags_for_name[:] # create a new list
 			name_tags.append(kind)
 			before_str = gajim.config.get('before_nickname')
+			before_str = helpers.from_one_line(before_str)
 			after_str = gajim.config.get('after_nickname')
+			after_str = helpers.from_one_line(after_str)
 			format = before_str + name + after_str + ' '
 			buffer.insert_with_tags_by_name(end_iter, format, *name_tags)
 
@@ -669,8 +801,18 @@ class ConversationTextview:
 			buffer.insert(end_iter, subject)
 			self.print_empty_line()
 
-	def print_real_text(self, text, text_tags = [], name = None):
+	def print_real_text(self, text, text_tags = [], name = None, xhtml = None):
 		'''this adds normal and special text. call this to add text'''
+		if xhtml:
+			try:
+				if name and (text.startswith('/me ') or text.startswith('/me\n')):
+					xhtml = xhtml.replace('/me', '<dfn>%s</dfn>'% (name,), 1)
+				self.tv.display_html(xhtml.encode('utf-8'))
+				return
+			except Exception, e:
+				gajim.log.debug(str("Error processing xhtml")+str(e))
+				gajim.log.debug(str("with |"+xhtml+"|"))
+
 		buffer = self.tv.get_buffer()
 		# /me is replaced by name if name is given
 		if name and (text.startswith('/me ') or text.startswith('/me\n')):
