@@ -1,7 +1,4 @@
-#!/bin/sh
-''':'
-exec python -OOt "$0" ${1+"$@"}
-' '''
+#!/usr/bin/env python
 ## history_manager.py
 ##
 ## Copyright (C) 2006 Nikos Kouremenos <kourem@gmail.com>
@@ -24,21 +21,20 @@ import sys
 import os
 import signal
 import gtk
-import gtk.glade
 import time
 import locale
 
+from common import i18n
 import exceptions
 import dialogs
 import gtkgui_helpers
 from common.logger import LOG_DB_PATH, constants
 
+#FIXME: constants should implement 2 way mappings 
+status = dict((constants.__dict__[i], i[5:].lower()) for i in \
+	constants.__dict__.keys() if i.startswith('SHOW_')) 
 from common import gajim
-from common import i18n
 from common import helpers
-_ = i18n._
-gtk.glade.bindtextdomain(i18n.APP, i18n.DIR)
-gtk.glade.textdomain(i18n.APP)
 
 # time, message, subject
 (
@@ -48,14 +44,17 @@ C_SUBJECT,
 C_NICKNAME
 ) = range(2, 6)
 
+
 try:
-	from pysqlite2 import dbapi2 as sqlite
+	import sqlite3 as sqlite # python 2.5
 except ImportError:
-	raise exceptions.PysqliteNotAvailable
+	try:
+		from pysqlite2 import dbapi2 as sqlite
+	except ImportError:
+		raise exceptions.PysqliteNotAvailable
 
 
 class HistoryManager:
-
 	def __init__(self):
 		path_to_file = os.path.join(gajim.DATA_DIR, 'pixmaps/gajim.png')
 		pix = gtk.gdk.pixbuf_new_from_file(path_to_file)
@@ -197,7 +196,9 @@ class HistoryManager:
 		gtk.main_quit()
 	
 	def _fill_jids_listview(self):
-		self.cur.execute('SELECT jid, jid_id FROM jids ORDER BY jid')
+		# get those jids that have at least one entry in logs
+		self.cur.execute('SELECT jid, jid_id FROM jids WHERE jid_id IN (SELECT '
+			'distinct logs.jid_id FROM logs) ORDER BY jid')
 		rows = self.cur.fetchall() # list of tupples: [(u'aaa@bbb',), (u'cc@dd',)]
 		for row in rows:
 			self.jids_already_in.append(row[0]) # jid
@@ -218,7 +219,7 @@ class HistoryManager:
 
 		list_of_rowrefs = []
 		for path in list_of_paths: # make them treerowrefs (it's needed)
-			 list_of_rowrefs.append(gtk.TreeRowReference(liststore, path))
+			list_of_rowrefs.append(gtk.TreeRowReference(liststore, path))
 		
 		for rowref in list_of_rowrefs: # FILL THE STORE, for all rows selected
 			path = rowref.get_path()
@@ -314,6 +315,7 @@ class HistoryManager:
 			except ValueError:
 				pass
 			else:
+				color = None
 				if kind in (constants.KIND_SINGLE_MSG_RECV,
 				constants.KIND_CHAT_MSG_RECV, constants.KIND_GC_MSG):
 					# it is the other side
@@ -329,11 +331,14 @@ class HistoryManager:
 						message = ''
 					else:
 						message = ' : ' + message 
-					message = helpers.get_uf_show(show) + message
-					
-				message = '<span foreground="%s">%s</span>' % (color,
-					gtkgui_helpers.escape_for_pango_markup(message))
-				self.logs_liststore.append((log_line_id, jid_id, time_, message,
+					message = helpers.get_uf_show(gajim.SHOW_LIST[show]) + message
+
+				message_ = '<span'
+				if color:
+					message_ += ' foreground="%s"' % color
+				message_ += '>%s</span>' % \
+					gtkgui_helpers.escape_for_pango_markup(message)
+				self.logs_liststore.append((log_line_id, jid_id, time_, message_,
 					subject, nickname))
 
 	def _fill_search_results_listview(self, text):
@@ -428,7 +433,7 @@ class HistoryManager:
 
 		list_of_rowrefs = []
 		for path in list_of_paths: # make them treerowrefs (it's needed)
-			 list_of_rowrefs.append(gtk.TreeRowReference(liststore, path))
+			list_of_rowrefs.append(gtk.TreeRowReference(liststore, path))
 		
 		for rowref in list_of_rowrefs:
 			path = rowref.get_path()
@@ -468,8 +473,8 @@ class HistoryManager:
 			except ValueError:
 				pass
 
-			file_.write(_('%(who)s on %(time)s said: %(message)s\n' % {'who': who,
-				'time': time_, 'message': message}))
+			file_.write(_('%(who)s on %(time)s said: %(message)s\n') % {'who': who,
+				'time': time_, 'message': message})
 	
 	def _delete_jid_logs(self, liststore, list_of_paths):
 		paths_len = len(list_of_paths)
@@ -481,7 +486,7 @@ class HistoryManager:
 			self.dialog.destroy()
 			list_of_rowrefs = []
 			for path in list_of_paths: # make them treerowrefs (it's needed)
-				 list_of_rowrefs.append(gtk.TreeRowReference(liststore, path))
+				list_of_rowrefs.append(gtk.TreeRowReference(liststore, path))
 
 			for rowref in list_of_rowrefs:
 				path = rowref.get_path()
@@ -523,7 +528,7 @@ class HistoryManager:
 			# delete rows from db that match log_line_id
 			list_of_rowrefs = []
 			for path in list_of_paths: # make them treerowrefs (it's needed)
-				 list_of_rowrefs.append(gtk.TreeRowReference(liststore, path))
+				list_of_rowrefs.append(gtk.TreeRowReference(liststore, path))
 
 			for rowref in list_of_rowrefs:
 				path = rowref.get_path()
