@@ -11,8 +11,6 @@
 ## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 ## GNU General Public License for more details.
 ##
-import gtk
-import gtk.glade
 import gtkgui_helpers
 
 from common import gajim
@@ -22,32 +20,21 @@ TYPE_CHAT = 'chat'
 TYPE_GC = 'gc'
 TYPE_PM = 'pm'
 
-####################
-# FIXME: Can't this stuff happen once?
-from common import i18n
-_ = i18n._
-APP = i18n.APP
-
 ####################
 
 class MessageControl:
 	'''An abstract base widget that can embed in the gtk.Notebook of a MessageWindow'''
 
-	def __init__(self, type_id, parent_win, widget_name, display_names, contact, account, resource = None):
-		'''The display_names argument is a two element tuple containing the desired
-		display name (pretty string) for the control in both singular and plural form'''
+	def __init__(self, type_id, parent_win, widget_name, contact, account, resource = None):
 		# dict { cb id : widget} 
 		# keep all registered callbacks of widgets, created by self.xml
 		self.handlers = {} 
 		self.type_id = type_id
 		self.parent_win = parent_win
 		self.widget_name = widget_name
-		self.display_names = display_names
 		self.contact = contact
 		self.account = account
-		self.hide_chat_buttons_always = False
 		self.hide_chat_buttons_current = False
-		self.nb_unread = 0
 		self.resource = resource
 
 		gajim.last_message_time[self.account][self.get_full_jid()] = 0
@@ -64,9 +51,9 @@ class MessageControl:
 	def set_control_active(self, state):
 		'''Called when the control becomes active (state is True)
 		or inactive (state is False)'''
-		pass  # Derived types MUST implement this method
+		pass  # Derived classes MUST implement this method
 
-	def allow_shutdown(self):
+	def allow_shutdown(self, method):
 		'''Called to check is a control is allowed to shutdown.
 		If a control is not in a suitable shutdown state this method
 		should return False'''
@@ -77,11 +64,7 @@ class MessageControl:
 		# NOTE: Derived classes MUST implement this
 		pass
 
-	def notify_on_new_messages(self):
-		# NOTE: Derived classes MUST implement this
-		return False
-
-	def repaint_themed_widgets(self, theme):
+	def repaint_themed_widgets(self):
 		pass # NOTE: Derived classes SHOULD implement this
 
 	def update_ui(self):
@@ -125,16 +108,16 @@ class MessageControl:
 		pass
 
 	def get_specific_unread(self):
-		n = 0
-		if gajim.awaiting_events[self.account].has_key(self.contact.jid):
-			n = len(gajim.awaiting_events[self.account][self.contact.jid])
-		return n
+		return len(gajim.events.get_events(self.account, self.contact.jid))
 
 	def send_message(self, message, keyID = '', type = 'chat',
-	chatstate = None, msg_id = None, composing_jep = None, resource = None):
-		'''Send the given message to the active tab'''
+	chatstate = None, msg_id = None, composing_jep = None, resource = None,
+	user_nick = None):
+		'''Send the given message to the active tab. Doesn't return None if error
+		'''
 		jid = self.contact.jid
 		# Send and update history
-		gajim.connections[self.account].send_message(jid, message, keyID,
-						type = type, chatstate = chatstate, msg_id = msg_id,
-						composing_jep = composing_jep, resource = self.resource)
+		return gajim.connections[self.account].send_message(jid, message, keyID,
+			type = type, chatstate = chatstate, msg_id = msg_id,
+			composing_jep = composing_jep, resource = self.resource,
+			user_nick = user_nick)
