@@ -4,7 +4,7 @@
 ##                         Vincent Hanquez <tab@snarc.org>
 ## Copyright (C) 2005 Yann Le Boulanger <asterix@lagaule.org>
 ##                    Vincent Hanquez <tab@snarc.org>
-##                    Nikos Kouremenos <nkour@jabber.org>
+##                    Nikos Kouremenos <kourem@gmail.com>
 ##                    Dimitur Kirov <dkirov@gmail.com>
 ##                    Travis Shirk <travis@pobox.com>
 ##                    Norman Rasmussen <norman@rasmussen.co.za>
@@ -22,7 +22,7 @@
 ##
 
 import gtk
-import gtk.glade
+import gobject
 
 import common
 import gtkgui_helpers
@@ -31,12 +31,6 @@ from chat_control import ChatControlBase
 
 from common import gajim
 
-####################
-# FIXME: Can't this stuff happen once?
-from common import i18n
-_ = i18n._
-APP = i18n.APP
-
 ####################
 
 class MessageWindow:
@@ -46,6 +40,13 @@ class MessageWindow:
 	# DND_TARGETS is the targets needed by drag_source_set and drag_dest_set
 	DND_TARGETS = [('GAJIM_TAB', 0, 81)]
 	hid = 0 # drag_data_received handler id
+	(
+		CLOSE_TAB_MIDDLE_CLICK,
+		CLOSE_ESC,
+		CLOSE_CLOSE_BUTTON,
+		CLOSE_COMMAND,
+		CLOSE_CTRL_KEY
+	) = range(5)
 	
 	def __init__(self, acct, type):
 		# A dictionary of dictionaries where _contacts[account][jid] == A MessageControl
@@ -110,6 +111,16 @@ class MessageWindow:
 		self.notebook.drag_dest_set(gtk.DEST_DEFAULT_ALL, self.DND_TARGETS,
 			gtk.gdk.ACTION_MOVE)
 
+	def change_account_name(self, old_name, new_name):
+		if self._controls.has_key(old_name):
+			self._controls[new_name] = self._controls[old_name]
+			del self._controls[old_name]
+		for ctrl in self.controls():
+			if ctrl.account == old_name:
+				ctrl.account = new_name
+		if self.account == old_name:
+			self.account = new_name
+
 	def get_num_controls(self):
 		n = 0
 		for dict in self._controls.values():
@@ -136,7 +147,7 @@ class MessageWindow:
 	def _on_window_delete(self, win, event):
 		# Make sure all controls are okay with being deleted
 		for ctrl in self.controls():
-			if not ctrl.allow_shutdown():
+			if not ctrl.allow_shutdown(self.CLOSE_CLOSE_BUTTON):
 				return True # halt the delete
 		return False
 
@@ -156,8 +167,17 @@ class MessageWindow:
 		fjid = control.get_full_jid()
 		self._controls[control.account][fjid] = control
 
-		if self.get_num_controls() > 1:
+		if self.get_num_controls() == 2:
+			# is first conversation_textview scrolled down ?
+			scrolled = False
+			first_widget = self.notebook.get_nth_page(0)
+			ctrl = self._widget_to_control(first_widget)
+			conv_textview = ctrl.conv_textview
+			if conv_textview.at_the_end():
+				scrolled = True
 			self.notebook.set_show_tabs(True)
+			if scrolled:
+				gobject.idle_add(conv_textview.scroll_to_end_iter)
 			self.alignment.set_property('top-padding', 2)
 
 		# Add notebook page and connect up to the tab's close button
@@ -186,7 +206,7 @@ class MessageWindow:
 			self.popup_menu(event)
 		elif event.button == 2: # middle click
 			ctrl = self._widget_to_control(child)
-			self.remove_tab(ctrl)
+			self.remove_tab(ctrl, self.CLOSE_TAB_MIDDLE_CLICK)
 
 	def _on_message_textview_mykeypress_event(self, widget, event_keyval,
 		event_keymod):
@@ -211,17 +231,19 @@ class MessageWindow:
 
 	def _on_close_button_clicked(self, button, control):
 		'''When close button is pressed: close a tab'''
-		self.remove_tab(control)
+		self.remove_tab(control, self.CLOSE_CLOSE_BUTTON)
 
 	def show_title(self, urgent = True, control = None):
 		'''redraw the window's title'''
 		unread = 0
 		for ctrl in self.controls():
 			if ctrl.type_id == message_control.TYPE_GC and not \
-				gajim.config.get('notify_on_all_muc_messages') and not \
-				ctrl.attention_flag:
+			gajim.config.get('notify_on_all_muc_messages') and not \
+			ctrl.attention_flag:
+				# count only pm messages
+				unread += ctrl.get_nb_unread_pm()
 				continue
-			unread += ctrl.nb_unread
+			unread += ctrl.get_nb_unread()
 
 		unread_str = ''
 		if unread > 1:
@@ -242,11 +264,17 @@ class MessageWindow:
 				name += '/' + control.resource
 
 		window_mode = gajim.interface.msg_win_mgr.mode
+
 		if self.get_num_controls() == 1:
 			label = name
 		elif window_mode == MessageWindowMgr.ONE_MSG_WINDOW_PERTYPE:
 			# Show the plural form since number of tabs > 1
-			label = control.display_names[1]
+			if self.type == 'chat':
+				label = _('Chats')
+			elif self.type == 'gc':
+				label = _('Group Chats')
+			else:
+				label = _('Private Chats')
 		else:
 			label = _('Messages')
 		title = _('%s - Gajim') % label
@@ -266,10 +294,11 @@ class MessageWindow:
 		ctrl_page = self.notebook.page_num(ctrl.widget)
 		self.notebook.set_current_page(ctrl_page)
 	
-	def remove_tab(self, ctrl, reason = None):
-		'''reason is only for gc (offline status message)'''
+	def remove_tab(self, ctrl, method, reason = None, force = False):
+		'''reason is only for gc (offline status message)
+		if force is True, do not ask any confirmation'''
 		# Shutdown the MessageControl
-		if not ctrl.allow_shutdown():
+		if not force and not ctrl.allow_shutdown(method):
 			return
 		if reason is not None: # We are leaving gc with a status message
 			ctrl.shutdown(reason)
@@ -277,9 +306,8 @@ class MessageWindow:
 			ctrl.shutdown()
 
 		# Update external state
-		if gajim.interface.systray_enabled:
-			gajim.interface.systray.remove_jid(ctrl.get_full_jid(), ctrl.account,
-				ctrl.type_id)
+		gajim.events.remove_events(ctrl.account, ctrl.get_full_jid,
+			types = ['printed_msg', 'chat', 'gc_msg'])
 		del gajim.last_message_time[ctrl.account][ctrl.get_full_jid()]
 
 		self.disconnect_tab_dnd(ctrl.widget)
@@ -290,28 +318,22 @@ class MessageWindow:
 		if len(self._controls[ctrl.account]) == 0:
 			del self._controls[ctrl.account]
 
-		# Notify a dupicate nick to update their banner and clear account display
-		for c in self.controls():
-			if c == self:
-				continue
-			if ctrl.contact.get_shown_name() == c.contact.get_shown_name():
-				c.draw_banner()
-
-		if self.get_num_controls() == 1: # we are going from two tabs to one
-			show_tabs_if_one_tab = gajim.config.get('tabs_always_visible')
-			self.notebook.set_show_tabs(show_tabs_if_one_tab)
-			if not show_tabs_if_one_tab:
-				self.alignment.set_property('top-padding', 0)
-			self.show_title()
-		elif self.get_num_controls() == 0:
+		if self.get_num_controls() == 0:
 			# These are not called when the window is destroyed like this, fake it
 			gajim.interface.msg_win_mgr._on_window_delete(self.window, None)
 			gajim.interface.msg_win_mgr._on_window_destroy(self.window)
 			# dnd clean up
 			self.notebook.disconnect(self.hid)
 			self.notebook.drag_dest_unset()
-
 			self.window.destroy()
+			return # don't show_title, we are dead
+		elif self.get_num_controls() == 1: # we are going from two tabs to one
+			show_tabs_if_one_tab = gajim.config.get('tabs_always_visible')
+			self.notebook.set_show_tabs(show_tabs_if_one_tab)
+			if not show_tabs_if_one_tab:
+				self.alignment.set_property('top-padding', 0)
+		self.show_title()
+
 			
 	def redraw_tab(self, ctrl, chatstate = None):
 		hbox = self.notebook.get_tab_label(ctrl.widget).get_children()[0]
@@ -416,6 +438,8 @@ class MessageWindow:
 		ind = self.notebook.get_current_page()
 		current = ind
 		found = False
+		first_composing_ind = -1 # id of first composing ctrl to switch to
+										# if no others controls have awaiting events
 		# loop until finding an unread tab or having done a complete cycle
 		while True: 
 			if forward == True: # look for the first unread tab on the right
@@ -426,15 +450,22 @@ class MessageWindow:
 				ind = ind - 1
 				if ind < 0:
 					ind = self.notebook.get_n_pages() - 1
-			if ind == current:
-				break # a complete cycle without finding an unread tab 
 			ctrl = self.get_control(ind, None)
-			if ctrl.nb_unread > 0:
+			if ctrl.get_nb_unread() > 0:
 				found = True
 				break # found
+			elif gajim.config.get('ctrl_tab_go_to_next_composing') : # Search for a composing contact
+				contact = ctrl.contact
+				if first_composing_ind == -1 and contact.chatstate == 'composing':
+				# If no composing contact found yet, check if this one is composing
+					first_composing_ind = ind
+			if ind == current:
+				break # a complete cycle without finding an unread tab 
 		if found:
 			self.notebook.set_current_page(ind)
-		else: # not found
+		elif first_composing_ind != -1:
+			self.notebook.set_current_page(first_composing_ind)
+		else: # not found and nobody composing
 			if forward: # CTRL + TAB
 				if current < (self.notebook.get_n_pages() - 1):
 					self.notebook.next_page()
@@ -466,8 +497,6 @@ class MessageWindow:
 	def _on_notebook_key_press(self, widget, event):
 		st = '1234567890' # alt+1 means the first tab (tab 0)
 		ctrl = self.get_active_control()
-		contact = ctrl.contact
-		jid = ctrl.contact.jid
 
 		# CTRL mask
 		if event.state & gtk.gdk.CONTROL_MASK:
@@ -477,9 +506,9 @@ class MessageWindow:
 			elif event.keyval == gtk.keysyms.Tab: # CTRL + TAB
 				self.move_to_next_unread_tab(True)
 			elif event.keyval == gtk.keysyms.F4: # CTRL + F4
-				self.remove_tab(ctrl)
+				self.remove_tab(ctrl, self.CLOSE_CTRL_KEY)
 			elif event.keyval == gtk.keysyms.w: # CTRL + W
-				self.remove_tab(ctrl)
+				self.remove_tab(ctrl, self.CLOSE_CTRL_KEY)
 
 		# MOD1 (ALT) mask
 		elif event.state & gtk.gdk.MOD1_MASK:
@@ -502,7 +531,7 @@ class MessageWindow:
 		# Close tab bindings
 		elif event.keyval == gtk.keysyms.Escape and \
 				gajim.config.get('escape_key_closes'): # Escape
-			self.remove_tab(ctrl)
+			self.remove_tab(ctrl, self.CLOSE_ESC)
 		else:
 			# If the active control has a message_textview pass the event to it
 			active_ctrl = self.get_active_control()
@@ -514,10 +543,10 @@ class MessageWindow:
 		'''Set tab label as drag source and connect the drag_data_get signal'''
 		tab_label = self.notebook.get_tab_label(child)
 		tab_label.dnd_handler = tab_label.connect('drag_data_get', 
-							  self.on_tab_label_drag_data_get_cb)
+			self.on_tab_label_drag_data_get_cb)
 		self.handlers[tab_label.dnd_handler] = tab_label
 		tab_label.drag_source_set(gtk.gdk.BUTTON1_MASK, self.DND_TARGETS,
-					 gtk.gdk.ACTION_MOVE)
+			gtk.gdk.ACTION_MOVE)
 		tab_label.page_num = self.notebook.page_num(child)
 
 	def on_tab_label_drag_data_get_cb(self, widget, drag_context, selection,
@@ -532,7 +561,6 @@ class MessageWindow:
 		source_page_num = int(selection.data)
 		dest_page_num, to_right = self.get_tab_at_xy(x, y)
 		source_child = self.notebook.get_nth_page(source_page_num)
-		source_tab_label = self.notebook.get_tab_label(source_child)
 		if dest_page_num != source_page_num:
 			self.notebook.reorder_child(source_child, dest_page_num)
 		
@@ -551,14 +579,14 @@ class MessageWindow:
 			tab_alloc = tab.get_allocation()
 			if horiz:
 				if (x >= tab_alloc.x) and \
-					   (x <= (tab_alloc.x + tab_alloc.width)):
+				(x <= (tab_alloc.x + tab_alloc.width)):
 					page_num = i
 					if x >= tab_alloc.x + (tab_alloc.width / 2.0):
 						to_right = True
 					break
 			else:
 				if (y >= tab_alloc.y) and \
-					   (y <= (tab_alloc.y + tab_alloc.height)):
+				(y <= (tab_alloc.y + tab_alloc.height)):
 					page_num = i
 				
 					if y > tab_alloc.y + (tab_alloc.height / 2.0):
@@ -599,15 +627,19 @@ class MessageWindowMgr:
 
 	def __init__(self):
 		''' A dictionary of windows; the key depends on the config:
-		 ONE_MSG_WINDOW_NEVER: The key is the contact JID
-		 ONE_MSG_WINDOW_ALWAYS: The key is MessageWindowMgr.MAIN_WIN 
-		 ONE_MSG_WINDOW_PERACCT: The key is the account name
-		 ONE_MSG_WINDOW_PERTYPE: The key is a message type constant'''
+		ONE_MSG_WINDOW_NEVER: The key is the contact JID
+		ONE_MSG_WINDOW_ALWAYS: The key is MessageWindowMgr.MAIN_WIN 
+		ONE_MSG_WINDOW_PERACCT: The key is the account name
+		ONE_MSG_WINDOW_PERTYPE: The key is a message type constant'''
 		self._windows = {}
 		# Map the mode to a int constant for frequent compares
 		mode = gajim.config.get('one_message_window')
 		self.mode = common.config.opt_one_window_types.index(mode)
-	
+
+	def change_account_name(self, old_name, new_name):
+		for win in self.windows():
+			win.change_account_name(old_name, new_name)
+
 	def _new_window(self, acct, type):
 		win = MessageWindow(acct, type)
 		# we track the lifetime of this window
@@ -641,13 +673,13 @@ class MessageWindowMgr:
 		if not gajim.config.get('saveposition'):
 			return
 			
-		if self.mode in (self.ONE_MSG_WINDOW_NEVER, self.ONE_MSG_WINDOW_ALWAYS):
+		if self.mode == self.ONE_MSG_WINDOW_ALWAYS:
 			size = (gajim.config.get('msgwin-width'),
 				gajim.config.get('msgwin-height'))
 		elif self.mode == self.ONE_MSG_WINDOW_PERACCT:
 			size = (gajim.config.get_per('accounts', acct, 'msgwin-width'),
 				gajim.config.get_per('accounts', acct, 'msgwin-height'))
-		elif self.mode == self.ONE_MSG_WINDOW_PERTYPE:
+		elif self.mode in (self.ONE_MSG_WINDOW_NEVER, self.ONE_MSG_WINDOW_PERTYPE):
 			if type == message_control.TYPE_PM:
 				type = message_control.TYPE_CHAT
 			opt_width = type + '-msgwin-width'
@@ -705,6 +737,7 @@ class MessageWindowMgr:
 			win_type = type
 			win_role = type
 		elif self.mode == self.ONE_MSG_WINDOW_NEVER:
+			win_type = type
 			win_role = contact.jid
 
 		win = None
@@ -792,6 +825,10 @@ class MessageWindowMgr:
 			pos_y_key = type + '-msgwin-y-position'
 			size_width_key = type + '-msgwin-width'
 			size_height_key = type + '-msgwin-height'
+		elif self.mode == self.ONE_MSG_WINDOW_NEVER:
+			type = msg_win.type
+			size_width_key = type + '-msgwin-width'
+			size_height_key = type + '-msgwin-height'
 
 		if acct:
 			gajim.config.set_per('accounts', acct, size_width_key, width)
